@@ -1,4 +1,4 @@
-# Copyright 2012-2014 Jussi Pakkanen
+# Copyright 2012-2014 The Meson development team
 
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -19,11 +19,7 @@ import mlog
 import copy, os
 
 
-known_basic_kwargs = {'ui_files': True,
-                      'moc_headers' : True,
-                      'qresources' : True,
-                      'moc_sources' : True,
-                      'install' : True,
+known_basic_kwargs = {'install' : True,
                       'c_pch' : True,
                       'cpp_pch' : True,
                       'c_args' : True,
@@ -72,11 +68,11 @@ class Build:
         self.data = []
         self.static_linker = None
         self.static_cross_linker = None
-        self.configure_files = []
         self.pot = []
         self.subprojects = {}
         self.pkgconfig_gens = []
         self.install_script = None
+        self.install_dirs = []
 
     def has_language(self, language):
         for i in self.compilers:
@@ -120,6 +116,9 @@ class Build:
     def get_configure_files(self):
         return self.configure_files
 
+    def get_install_subdirs(self):
+        return self.install_dirs
+
     def get_global_args(self, compiler):
         return self.global_args.get(compiler.get_language(), [])
 
@@ -203,14 +202,15 @@ class BuildTarget():
     def process_sourcelist(self, sources):
         if not isinstance(sources, list):
             sources = [sources]
+        added_sources = {} # If the same source is defined multiple times, use it only once.
         for s in sources:
             # Holder unpacking. Ugly.
-            if hasattr(s, 'glist'):
-                s = s.glist
             if hasattr(s, 'held_object'):
                 s = s.held_object
             if isinstance(s, str):
-                self.sources.append(s)
+                if not s in added_sources:
+                    self.sources.append(s)
+                    added_sources[s] = True
             elif isinstance(s, GeneratedList) or isinstance(s, CustomTarget):
                 self.generated.append(s)
             else:
@@ -232,9 +232,7 @@ class BuildTarget():
             d = [d]
         newd = []
         for i in d:
-            if hasattr(i, 'el'):
-                newd.append(i.el)
-            elif hasattr(i, 'held_object'):
+            if hasattr(i, 'held_object'):
                 newd.append(i.held_object)
             else:
                 newd.append(i)
@@ -416,8 +414,8 @@ class BuildTarget():
 
     def add_external_deps(self, deps):
         for dep in deps:
-            if hasattr(dep, 'el'):
-                dep = dep.el
+            if hasattr(dep, 'held_object'):
+                dep = dep.held_object
             if not isinstance(dep, dependencies.Dependency):
                 raise InvalidArguments('Argument is not an external dependency')
             self.external_deps.append(dep)
@@ -471,8 +469,8 @@ class BuildTarget():
         ids = []
         for a in args:
             # FIXME same hack, forcibly unpack from holder.
-            if hasattr(a, 'includedirs'):
-                a = a.includedirs
+            if hasattr(a, 'held_object'):
+                a = a.held_object
             if not isinstance(a, IncludeDirs):
                 raise InvalidArguments('Include directory to be added is not an include directory object.')
             ids.append(a)
@@ -495,15 +493,12 @@ class Generator():
     def __init__(self, args, kwargs):
         if len(args) != 1:
             raise InvalidArguments('Generator requires one and only one positional argument')
-
-        if hasattr(args[0], 'held_object'):
-            exe = args[0].held_object
-            if not isinstance(exe, Executable):
-                raise InvalidArguments('First generator argument must be an executable.')
-        elif hasattr(args[0], 'ep'):
-            exe = args[0].ep
-        else:
-            raise InvalidArguments('First generator argument must be an executable object.')
+        
+        exe = args[0]
+        if hasattr(exe, 'held_object'):
+            exe = exe.held_object
+        if not isinstance(exe, Executable) and not isinstance(exe, dependencies.ExternalProgram):
+            raise InvalidArguments('First generator argument must be an executable.')
         self.exe = exe
         self.process_kwargs(kwargs)
 
@@ -551,8 +546,8 @@ class Generator():
 
 class GeneratedList():
     def __init__(self, generator):
-        if hasattr(generator, 'generator'):
-            generator = generator.generator
+        if hasattr(generator, 'held_object'):
+            generator = generator.held_object
         self.generator = generator
         self.infilelist = []
         self.outfilelist = []
@@ -669,7 +664,8 @@ class CustomTarget:
                     'output' : True,
                     'command' : True,
                     'install' : True,
-                    'install_dir' : True}
+                    'install_dir' : True,
+                    'build_always' : True}
     def __init__(self, name, subdir, kwargs):
         self.name = name
         self.subdir = subdir
@@ -706,8 +702,6 @@ class CustomTarget:
             cmd = [cmd]
         final_cmd = []
         for i, c in enumerate(cmd):
-            if hasattr(c, 'ep'):
-                c = c.ep
             if hasattr(c, 'held_object'):
                 c = c.held_object
             if isinstance(c, str):
@@ -744,6 +738,9 @@ class CustomTarget:
                 raise InvalidArguments('"install_dir" must be a string.')
         else:
             self.install = False
+        self.build_always = kwargs.get('build_always', False)
+        if not isinstance(self.build_always, bool):
+            raise InvalidArguments('Argument build_always must be a boolean.')
 
     def get_basename(self):
         return self.name
