@@ -1,6 +1,4 @@
-#!/usr/bin/env python3
-
-# Copyright 2013-2014 Jussi Pakkanen
+# Copyright 2013-2014 The Meson development team
 
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -23,9 +21,8 @@
 
 import os, stat, glob, subprocess, shutil
 from coredata import MesonException
-import environment
 import mlog
-from environment import is_windows
+import mesonlib
 
 class DependencyException(MesonException):
     def __init__(self, *args, **kwargs):
@@ -60,24 +57,29 @@ class Dependency():
     def get_name(self):
         return self.name
 
-    # Rules for commands to execute before compilation
-    # such as Qt's moc preprocessor.
-    def get_generate_rules(self):
-        return []
-
     def get_exe_args(self):
         return []
 
-class PkgConfigDependency(Dependency):
-    pkgconfig_found = False
+    def need_threads(self):
+        return False
 
-    def __init__(self, name, required):
+class PkgConfigDependency(Dependency):
+    pkgconfig_found = None
+
+    def __init__(self, name, kwargs):
+        required = kwargs.get('required', True)
         Dependency.__init__(self)
         self.name = name
-        if not PkgConfigDependency.pkgconfig_found:
+        if PkgConfigDependency.pkgconfig_found is None:
             self.check_pkgconfig()
 
         self.is_found = False
+        if not PkgConfigDependency.pkgconfig_found:
+            if required:
+                raise DependencyException('Pkg-config not found.')
+            self.cargs = []
+            self.libs = []
+            return
         p = subprocess.Popen(['pkg-config', '--modversion', name], stdout=subprocess.PIPE,
                                  stderr=subprocess.PIPE)
         out = p.communicate()[0]
@@ -89,9 +91,19 @@ class PkgConfigDependency(Dependency):
             self.cargs = []
             self.libs = []
         else:
-            mlog.log('Dependency', mlog.bold(name), 'found:', mlog.green('YES'))
-            self.is_found = True
             self.modversion = out.decode().strip()
+            mlog.log('Dependency', mlog.bold(name), 'found:', mlog.green('YES'), self.modversion)
+            version_requirement = kwargs.get('version', None)
+            if version_requirement is None:
+                self.is_found = True
+            else:
+                if not isinstance(version_requirement, str):
+                    raise DependencyException('Version argument must be string.')
+                self.is_found = mesonlib.version_compare(self.modversion, version_requirement)
+                if not self.is_found and required:
+                    raise DependencyException('Invalid version of a dependency, needed %s %s found %s.' % (name, version_requirement, self.modversion))
+            if not self.is_found:
+                return
             p = subprocess.Popen(['pkg-config', '--cflags', name], stdout=subprocess.PIPE,
                                  stderr=subprocess.PIPE)
             out = p.communicate()[0]
@@ -116,14 +128,92 @@ class PkgConfigDependency(Dependency):
         return self.libs
 
     def check_pkgconfig(self):
-        p = subprocess.Popen(['pkg-config', '--version'], stdout=subprocess.PIPE,
-                             stderr=subprocess.PIPE)
+        try:
+            p = subprocess.Popen(['pkg-config', '--version'], stdout=subprocess.PIPE,
+                                 stderr=subprocess.PIPE)
+            out = p.communicate()[0]
+            if p.returncode == 0:
+                mlog.log('Found pkg-config:', mlog.bold(shutil.which('pkg-config')),
+                         '(%s)' % out.decode().strip())
+                PkgConfigDependency.pkgconfig_found = True
+                return
+        except Exception:
+            pass
+        PkgConfigDependency.pkgconfig_found = False
+        mlog.log('Found Pkg-config:', mlog.red('NO'))
+
+    def found(self):
+        return self.is_found
+
+class WxDependency(Dependency):
+    wx_found = None
+
+    def __init__(self, kwargs):
+        Dependency.__init__(self)
+        if WxDependency.wx_found is None:
+            self.check_wxconfig()
+
+        if not WxDependency.wx_found:
+            raise DependencyException('Wx-config not found.')
+        self.is_found = False
+        p = subprocess.Popen([self.wxc, '--version'], stdout=subprocess.PIPE,
+                                 stderr=subprocess.PIPE)
         out = p.communicate()[0]
         if p.returncode != 0:
-            raise RuntimeError('Pkg-config executable not found.')
-        mlog.log('Found pkg-config version:', mlog.bold(out.decode().strip()),
-                 '(%s)' % shutil.which('pkg-config'))
-        PkgConfigDependency.pkgconfig_found = True
+            mlog.log('Dependency wxwidgets found:', mlog.red('NO'))
+            self.cargs = []
+            self.libs = []
+        else:
+            self.modversion = out.decode().strip()
+            version_req = kwargs.get('version', None)
+            if version_req is not None:
+                if not mesonlib.version_compare(self.modversion, version_req):
+                    mlog.log('Wxwidgets version %s does not fullfill requirement %s' %\
+                             (self.modversion, version_req))
+                    return
+            mlog.log('Dependency wxwidgets found:', mlog.green('YES'))
+            self.is_found = True
+            # wx-config seems to have a cflags as well but since it requires C++,
+            # this should be good, at least for now.
+            p = subprocess.Popen([self.wxc, '--cxxflags'], stdout=subprocess.PIPE,
+                                 stderr=subprocess.PIPE)
+            out = p.communicate()[0]
+            if p.returncode != 0:
+                raise RuntimeError('Could not generate cargs for wxwidgets.')
+            self.cargs = out.decode().split()
+
+            p = subprocess.Popen([self.wxc, '--libs'], stdout=subprocess.PIPE,
+                                 stderr=subprocess.PIPE)
+            out = p.communicate()[0]
+            if p.returncode != 0:
+                raise RuntimeError('Could not generate libs for wxwidgets.')
+            self.libs = out.decode().split()
+
+    def get_modversion(self):
+        return self.modversion
+
+    def get_compile_args(self):
+        return self.cargs
+
+    def get_link_args(self):
+        return self.libs
+
+    def check_wxconfig(self):
+        for wxc in ['wx-config-3.0', 'wx-config']:
+            try:
+                p = subprocess.Popen([wxc, '--version'], stdout=subprocess.PIPE,
+                                     stderr=subprocess.PIPE)
+                out = p.communicate()[0]
+                if p.returncode == 0:
+                    mlog.log('Found wx-config:', mlog.bold(shutil.which(wxc)),
+                             '(%s)' % out.decode().strip())
+                    self.wxc = wxc
+                    WxDependency.wx_found = True
+                    return
+            except Exception:
+                pass
+        WxDependency.wxconfig_found = False
+        mlog.log('Found wx-config:', mlog.red('NO'))
 
     def found(self):
         return self.is_found
@@ -131,17 +221,21 @@ class PkgConfigDependency(Dependency):
 class ExternalProgram():
     def __init__(self, name, fullpath=None, silent=False, search_dir=None):
         self.name = name
+        self.fullpath = None
         if fullpath is not None:
-            self.fullpath = [fullpath]
+            if not isinstance(fullpath, list):
+                self.fullpath = [fullpath]
+            else:
+                self.fullpath = fullpath
         else:
             self.fullpath = [shutil.which(name)]
             if self.fullpath[0] is None and search_dir is not None:
                 trial = os.path.join(search_dir, name)
                 suffix = os.path.splitext(trial)[-1].lower()[1:]
-                if environment.is_windows() and (suffix == 'exe' or suffix == 'com'\
+                if mesonlib.is_windows() and (suffix == 'exe' or suffix == 'com'\
                                           or suffix == 'bat'):
                     self.fullpath = [trial]
-                elif not environment.is_windows() and os.access(trial, os.X_OK):
+                elif not mesonlib.is_windows() and os.access(trial, os.X_OK):
                     self.fullpath = [trial]
                 else:
                     # Now getting desperate. Maybe it is a script file that is a) not chmodded
@@ -150,8 +244,10 @@ class ExternalProgram():
                         first_line = open(trial).readline().strip()
                         if first_line.startswith('#!'):
                             commands = first_line[2:].split('#')[0].strip().split()
-                            if environment.is_windows():
+                            if mesonlib.is_windows():
                                 commands[0] = commands[0].split('/')[-1] # Windows does not have /usr/bin.
+                                if commands[0] == 'env':
+                                    commands = commands[1:]
                             self.fullpath = commands + [trial]
                     except Exception:
                         pass
@@ -189,18 +285,12 @@ class ExternalLibrary(Dependency):
             return [self.fullpath]
         return []
 
-def find_external_dependency(name, kwargs):
-    required = kwargs.get('required', True)
-    if not isinstance(required, bool):
-        raise DependencyException('Keyword "required" must be a boolean.')
-    if name in packages:
-        dep = packages[name](kwargs)
-        if required and not dep.found():
-            raise DependencyException('Dependency "%s" not found' % name)
-        return dep
-    return PkgConfigDependency(name, required)
-
 class BoostDependency(Dependency):
+    # Some boost libraries have different names for
+    # their sources and libraries. This dict maps
+    # between the two.
+    name2lib = {'test' : 'unit_test_framework'}
+
     def __init__(self, kwargs):
         Dependency.__init__(self)
         self.name = 'boost'
@@ -285,7 +375,7 @@ class BoostDependency(Dependency):
     def detect_lib_modules(self):
         globber = 'libboost_*.so' # FIXME, make platform independent.
         if self.boost_root is None:
-            libdirs = environment.get_library_dirs()
+            libdirs = mesonlib.get_library_dirs()
         else:
             libdirs = [os.path.join(self.boost_root, 'lib')]
         for libdir in libdirs:
@@ -307,23 +397,34 @@ class BoostDependency(Dependency):
             # pkg-config returns.
             args.append('-L' + os.path.join(self.boost_root, 'lib'))
         for module in self.requested_modules:
+            module = BoostDependency.name2lib.get(module, module)
             if module in self.lib_modules or module in self.lib_modules_mt:
                 linkcmd = '-lboost_' + module
                 args.append(linkcmd)
+                # FIXME a hack, but Boost's testing framework has a lot of
+                # different options and it's hard to determine what to do
+                # without feedback from actual users. Update this
+                # as we get more bug reports.
+                if module == 'unit_testing_framework':
+                    args.append('-lboost_test_exec_monitor')
             elif module + '-mt' in self.lib_modules_mt:
                 linkcmd = '-lboost_' + module + '-mt'
                 args.append(linkcmd)
+                if module == 'unit_testing_framework':
+                    args.append('-lboost_test_exec_monitor-mt')
         return args
 
     def get_sources(self):
         return []
 
+    def need_threads(self):
+        return 'thread' in self.requested_modules
+
 class GTestDependency(Dependency):
     def __init__(self, kwargs):
         Dependency.__init__(self)
         self.main = kwargs.get('main', False)
         self.name = 'gtest'
-        self.libdir = '/usr/lib'
         self.libname = 'libgtest.so'
         self.libmain_name = 'libgtest_main.so'
         self.include_dir = '/usr/include'
@@ -337,9 +438,15 @@ class GTestDependency(Dependency):
         return self.is_found
 
     def detect(self):
-        libname = os.path.join(self.libdir, self.libname)
-        mainname = os.path.join(self.libdir, self.libmain_name)
-        if os.path.exists(libname) and os.path.exists(mainname):
+        trial_dirs = mesonlib.get_library_dirs()
+        glib_found = False
+        gmain_found = False
+        for d in trial_dirs:
+            if os.path.isfile(os.path.join(d, self.libname)):
+                glib_found = True
+            if os.path.isfile(os.path.join(d, self.libmain_name)):
+                gmain_found = True
+        if glib_found and gmain_found:
             self.is_found = True
             self.compile_args = []
             self.link_args = ['-lgtest']
@@ -359,8 +466,6 @@ class GTestDependency(Dependency):
         else:
             mlog.log('Dependency GTest found:', mlog.red('NO'))
             self.is_found = False
-        if self.is_found:
-            self.link_args.append('-lpthread')
         return self.is_found
 
     def get_compile_args(self):
@@ -377,38 +482,47 @@ class GTestDependency(Dependency):
     def get_sources(self):
         return self.sources
 
+    def need_threads(self):
+        return True
+
 class GMockDependency(Dependency):
     def __init__(self, kwargs):
         Dependency.__init__(self)
         # GMock may be a library or just source.
         # Work with both.
         self.name = 'gmock'
-        self.libdir = '/usr/lib'
         self.libname = 'libgmock.so'
-        self.src_include_dir = '/usr/src/gmock'
-        self.src_dir = '/usr/src/gmock/src'
-        self.all_src = os.path.join(self.src_dir, 'gmock-all.cc')
-        self.main_src = os.path.join(self.src_dir, 'gmock_main.cc')
-        fname = os.path.join(self.libdir, self.libname)
-        if os.path.exists(fname):
+        trial_dirs = mesonlib.get_library_dirs()
+        gmock_found = False
+        for d in trial_dirs:
+            if os.path.isfile(os.path.join(d, self.libname)):
+                gmock_found = True
+        if gmock_found:
             self.is_found = True
             self.compile_args = []
             self.link_args = ['-lgmock']
             self.sources = []
             mlog.log('Dependency GMock found:', mlog.green('YES'), '(prebuilt)')
-        elif os.path.exists(self.src_dir):
-            self.is_found = True
-            self.compile_args = ['-I' + self.src_include_dir]
-            self.link_args = []
-            if kwargs.get('main', False):
-                self.sources = [self.all_src, self.main_src]
-            else:
-                self.sources = [self.all_src]
-            mlog.log('Dependency GMock found:', mlog.green('YES'), '(building self)')
+            return
 
-        else:
-            mlog.log('Dependency GMock found:', mlog.red('NO'))
-            self.is_found = False
+        for d in ['/usr/src/gmock/src', '/usr/src/gmock']:
+            if os.path.exists(d):
+                self.is_found = True
+                # Yes, we need both because there are multiple
+                # versions of gmock that do different things.
+                self.compile_args = ['-I/usr/src/gmock', '-I/usr/src/gmock/src']
+                self.link_args = []
+                all_src = os.path.join(d, 'gmock-all.cc')
+                main_src = os.path.join(d, 'gmock_main.cc')
+                if kwargs.get('main', False):
+                    self.sources = [all_src, main_src]
+                else:
+                    self.sources = [all_src]
+                mlog.log('Dependency GMock found:', mlog.green('YES'), '(building self)')
+                return
+
+        mlog.log('Dependency GMock found:', mlog.red('NO'))
+        self.is_found = False
 
     def get_version(self):
         return '1.something_maybe'
@@ -425,8 +539,6 @@ class GMockDependency(Dependency):
     def found(self):
         return self.is_found
 
-qt5toolinfo_printed = False
-
 class Qt5Dependency(Dependency):
     def __init__(self, kwargs):
         Dependency.__init__(self)
@@ -437,88 +549,9 @@ class Qt5Dependency(Dependency):
         if isinstance(mods, str):
             mods = [mods]
         for module in mods:
-            self.modules.append(PkgConfigDependency('Qt5' + module, False))
+            self.modules.append(PkgConfigDependency('Qt5' + module, kwargs))
         if len(self.modules) == 0:
             raise DependencyException('No Qt5 modules specified.')
-        if not qt5toolinfo_printed:
-            mlog.log('Dependency Qt5 tools:')
-        self.find_exes()
-
-    def find_exes(self):
-        # The binaries have different names on different
-        # distros. Joy.
-        global qt5toolinfo_printed
-        self.moc = ExternalProgram('moc', silent=True)
-        if not self.moc.found():
-            self.moc = ExternalProgram('moc-qt5', silent=True)
-        self.uic = ExternalProgram('uic', silent=True)
-        if not self.uic.found():
-            self.uic = ExternalProgram('uic-qt5', silent=True)
-        self.rcc = ExternalProgram('rcc', silent=True)
-        if not self.rcc.found():
-            self.rcc = ExternalProgram('rcc-qt5', silent=True)
-
-        # Moc, uic and rcc write their version strings to stderr.
-        # Moc and rcc return a non-zero result when doing so.
-        # What kind of an idiot thought that was a good idea?
-        if self.moc.found():
-            mp = subprocess.Popen(self.moc.get_command() + ['-v'],
-                                  stdout=subprocess.PIPE, stderr=subprocess.PIPE)
-            (stdout, stderr) = mp.communicate()
-            stdout = stdout.decode().strip()
-            stderr = stderr.decode().strip()
-            if 'Qt 5' in stderr:
-                moc_ver = stderr
-            elif '5.' in stdout:
-                moc_ver = stdout
-            else:
-                raise DependencyException('Moc preprocessor is not for Qt 5. Output:\n%s\n%s' %
-                                          (stdout, stderr))
-            if not qt5toolinfo_printed:
-                mlog.log(' moc:', mlog.green('YES'), '(%s, %s)' % \
-                         (' '.join(self.moc.fullpath), moc_ver.split()[-1]))
-        else:
-            if not qt5toolinfo_printed:
-                mlog.log(' moc:', mlog.red('NO'))
-        if self.uic.found():
-            up = subprocess.Popen(self.uic.get_command() + ['-v'],
-                                  stdout=subprocess.PIPE, stderr=subprocess.PIPE)
-            (stdout, stderr) = up.communicate()
-            stdout = stdout.decode().strip()
-            stderr = stderr.decode().strip()
-            if 'version 5.' in stderr:
-                uic_ver = stderr
-            elif '5.' in stdout:
-                uic_ver = stdout
-            else:
-                raise DependencyException('Uic compiler is not for Qt 5. Output:\n%s\n%s' %
-                                          (stdout, stderr))
-            if not qt5toolinfo_printed:
-                mlog.log(' uic:', mlog.green('YES'), '(%s, %s)' % \
-                         (' '.join(self.uic.fullpath), uic_ver.split()[-1]))
-        else:
-            if not qt5toolinfo_printed:
-                mlog.log(' uic:', mlog.red('NO'))
-        if self.rcc.found():
-            rp = subprocess.Popen(self.rcc.get_command() + ['-v'],
-                                  stdout=subprocess.PIPE, stderr=subprocess.PIPE)
-            (stdout, stderr) = rp.communicate()
-            stdout = stdout.decode().strip()
-            stderr = stderr.decode().strip()
-            if 'version 5.' in stderr:
-                rcc_ver = stderr
-            elif '5.' in stdout:
-                rcc_ver = stdout
-            else:
-                raise DependencyException('Rcc compiler is not for Qt 5. Output:\n%s\n%s' %
-                                          (stdout, stderr))
-            if not qt5toolinfo_printed:
-                mlog.log(' rcc:', mlog.green('YES'), '(%s, %s)'\
-                        % (' '.join(self.rcc.fullpath), rcc_ver.split()[-1]))
-        else:
-            if not qt5toolinfo_printed:
-                mlog.log(' rcc:', mlog.red('NO'))
-        qt5toolinfo_printed = True
 
     def get_version(self):
         return self.modules[0].get_version()
@@ -539,32 +572,11 @@ class Qt5Dependency(Dependency):
         return args
 
     def found(self):
-        if not self.moc.found():
-            return False
-        if not self.uic.found():
-            return False
-        if not self.rcc.found():
-            return False
         for i in self.modules:
             if not i.found():
                 return False
         return True
 
-    def get_generate_rules(self):
-        moc_rule = CustomRule(self.moc.get_command() + ['$mocargs', '@INFILE@', '-o', '@OUTFILE@'],
-                              'moc_@BASENAME@.cpp', 'moc_headers', 'moc_hdr_compile',
-                              'Compiling header @INFILE@ with the moc preprocessor')
-        mocsrc_rule = CustomRule(self.moc.get_command() + ['$mocargs', '@INFILE@', '-o', '@OUTFILE@'],
-                              '@BASENAME@.moc', 'moc_sources', 'moc_src_compile',
-                              'Compiling source @INFILE@ with the moc preprocessor')
-        ui_rule = CustomRule(self.uic.get_command() + ['@INFILE@', '-o', '@OUTFILE@'],
-                              'ui_@BASENAME@.h', 'ui_files', 'ui_compile',
-                              'Compiling @INFILE@ with the ui compiler')
-        rrc_rule = CustomRule(self.rcc.get_command() + ['@INFILE@', '-o', '@OUTFILE@',
-                               '${rcc_args}'], '@BASENAME@.cpp','qresources',
-                              'rc_compile', 'Compiling @INFILE@ with the rrc compiler')
-        return [moc_rule, mocsrc_rule, ui_rule, rrc_rule]
-
     def get_exe_args(self):
         # Qt5 seems to require this always.
         # Fix this to be more portable, especially to MSVC.
@@ -578,9 +590,15 @@ class GnuStepDependency(Dependency):
 
     def detect(self):
         confprog = 'gnustep-config'
-        gp = subprocess.Popen([confprog, '--help'],
-                              stdout=subprocess.PIPE, stderr=subprocess.PIPE)
-        gp.communicate()
+        try:
+            gp = subprocess.Popen([confprog, '--help'],
+                                  stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+            
+            gp.communicate()
+        except FileNotFoundError:
+            self.args = None
+            mlog.log('Dependency GnuStep found:', mlog.red('NO'), '(no gnustep-config)')
+            return
         if gp.returncode != 0:
             self.args = None
             mlog.log('Dependency GnuStep found:', mlog.red('NO'))
@@ -655,7 +673,86 @@ class AppleFrameworks(Dependency):
         return args
 
     def found(self):
-        return environment.is_osx()
+        return mesonlib.is_osx()
+
+# There are three different ways of depending on SDL2:
+# sdl2-config, pkg-config and OSX framework
+class SDL2Dependency(Dependency):
+    def __init__(self, kwargs):
+        Dependency.__init__(self)
+        self.is_found = False
+        self.cargs = []
+        self.linkargs = []
+        sdlconf = shutil.which('sdl2-config')
+        if sdlconf:
+            pc = subprocess.Popen(['sdl2-config', '--cflags'], stdout=subprocess.PIPE, stderr=subprocess.DEVNULL)
+            (stdo, _) = pc.communicate()
+            self.cargs = stdo.decode().strip().split()
+            pc = subprocess.Popen(['sdl2-config', '--libs'], stdout=subprocess.PIPE, stderr=subprocess.DEVNULL)
+            (stdo, _) = pc.communicate()
+            self.linkargs = stdo.decode().strip().split()
+            self.is_found = True
+            mlog.log('Dependency', mlog.bold('sdl2'), 'found:', mlog.green('YES'), '(%s)' % sdlconf)
+            return
+        try:
+            pcdep = PkgConfigDependency('sdl2', kwargs)
+            if pcdep.found():
+                self.is_found = True
+                self.cargs = pcdep.get_compile_args()
+                self.linkargs = pcdep.get_link_args()
+                return
+        except Exception:
+            pass
+        if mesonlib.is_osx():
+            fwdep = ExtraFrameworkDependency('sdl2', kwargs.get('required', True))
+            if fwdep.found():
+                self.is_found = True
+                self.cargs = fwdep.get_compile_args()
+                self.linkargs = fwdep.get_link_args()
+                return
+        mlog.log('Dependency', mlog.bold('sdl2'), 'found:', mlog.red('NO'))
+
+    def get_compile_args(self):
+        return self.cargs
+
+    def get_link_args(self):
+        return self.linkargs
+
+    def found(self):
+        return self.is_found
+
+class ExtraFrameworkDependency(Dependency):
+    def __init__(self, name, required):
+        Dependency.__init__(self)
+        self.name = None
+        self.detect(name)
+        if self.found():
+            mlog.log('Dependency', mlog.bold(name), 'found:', mlog.green('YES'), os.path.join(self.path, self.name))
+        else:
+            mlog.log('Dependency', name, 'found:', mlog.red('NO'))
+
+    def detect(self, name):
+        lname = name.lower()
+        paths = ['/Library/Frameworks']
+        for p in paths:
+            for d in os.listdir(p):
+                fullpath = os.path.join(p, d)
+                if lname != d.split('.')[0].lower():
+                    continue
+                if not stat.S_ISDIR(os.stat(fullpath).st_mode):
+                    continue
+                self.path = p
+                self.name = d
+                return
+
+    def get_compile_args(self):
+        return ['-I' + os.path.join(self.path, self.name, 'Headers')]
+
+    def get_link_args(self):
+        return ['-F' + self.path, '-framework', self.name.split('.')[0]]
+
+    def found(self):
+        return self.name is not None
 
 def get_dep_identifier(name, kwargs):
     elements = [name]
@@ -666,13 +763,38 @@ def get_dep_identifier(name, kwargs):
         elements.append(module)
     return '/'.join(elements)
 
+def find_external_dependency(name, kwargs):
+    required = kwargs.get('required', True)
+    if not isinstance(required, bool):
+        raise DependencyException('Keyword "required" must be a boolean.')
+    lname = name.lower()
+    if lname in packages:
+        dep = packages[lname](kwargs)
+        if required and not dep.found():
+            raise DependencyException('Dependency "%s" not found' % name)
+        return dep
+    pkg_exc = None
+    pkgdep = None
+    try:
+        pkgdep = PkgConfigDependency(name, kwargs)
+        if pkgdep.found():
+            return pkgdep
+    except Exception as e:
+        pkg_exc = e
+    if mesonlib.is_osx():
+        return ExtraFrameworkDependency(name, required)
+    if pkg_exc is not None:
+        raise pkg_exc
+    return pkgdep
+
 # This has to be at the end so the classes it references
 # are defined.
 packages = {'boost': BoostDependency,
             'gtest': GTestDependency,
             'gmock': GMockDependency,
             'qt5': Qt5Dependency,
-            'Qt5': Qt5Dependency, # Qt people sure do love their upper case.
             'gnustep': GnuStepDependency,
             'appleframeworks': AppleFrameworks,
+            'wxwidgets' : WxDependency,
+            'sdl2' : SDL2Dependency,
             }
