@@ -1,4 +1,4 @@
-# Copyright 2012-2014 Jussi Pakkanen
+# Copyright 2012-2015 The Meson development team
 
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -19,7 +19,10 @@ import dependencies
 import mlog
 import build
 import optinterpreter
-import os, sys, platform, subprocess, shutil, uuid
+import wrap
+import mesonlib
+import os, sys, platform, subprocess, shutil, uuid, re
+import importlib
 
 class InterpreterException(coredata.MesonException):
     pass
@@ -167,56 +170,65 @@ class ConfigurationDataHolder(InterpreterObject):
 # Interpreter objects can not be pickled so we must have
 # these wrappers.
 
+class DependencyHolder(InterpreterObject):
+    def __init__(self, dep):
+        InterpreterObject.__init__(self)
+        self.held_object = dep
+        self.methods.update({'found' : self.found_method})
+
+    def found_method(self, args, kwargs):
+        return self.held_object.found()
+
 class ExternalProgramHolder(InterpreterObject):
     def __init__(self, ep):
         InterpreterObject.__init__(self)
-        self.ep = ep
+        self.held_object = ep
         self.methods.update({'found': self.found_method})
 
     def found_method(self, args, kwargs):
         return self.found()
 
     def found(self):
-        return self.ep.found()
+        return self.held_object.found()
 
     def get_command(self):
-        return self.ep.fullpath
+        return self.held_object.fullpath
 
     def get_name(self):
-        return self.ep.name
+        return self.held_object.name
 
 class ExternalLibraryHolder(InterpreterObject):
     def __init__(self, el):
         InterpreterObject.__init__(self)
-        self.el = el
+        self.held_object = el
         self.methods.update({'found': self.found_method})
 
     def found(self):
-        return self.el.found()
+        return self.held_object.found()
 
     def found_method(self, args, kwargs):
         return self.found()
 
     def get_filename(self):
-        return self.el.fullpath
+        return self.held_object.fullpath
 
     def get_name(self):
-        return self.el.name
+        return self.held_object.name
 
     def get_compile_args(self):
-        return self.el.get_compile_args()
+        return self.held_object.get_compile_args()
 
     def get_link_args(self):
-        return self.el.get_link_args()
+        return self.held_object.get_link_args()
 
     def get_exe_args(self):
-        return self.el.get_exe_args()
+        return self.held_object.get_exe_args()
 
 class GeneratorHolder(InterpreterObject):
     def __init__(self, interpreter, args, kwargs):
         super().__init__()
         self.interpreter = interpreter
-        self.generator = build.Generator(args, kwargs)
+        self.held_object = build.Generator(args, kwargs)
         self.methods.update({'process' : self.process_method})
 
     def process_method(self, args, kwargs):
@@ -234,12 +246,15 @@ class GeneratorHolder(InterpreterObject):
         return gl
 
 class GeneratedListHolder(InterpreterObject):
-    def __init__(self, generator):
+    def __init__(self, arg1):
         super().__init__()
-        self.glist = build.GeneratedList(generator)
+        if isinstance(arg1, GeneratorHolder):
+            self.held_object = build.GeneratedList(arg1.held_object)
+        else:
+            self.held_object = arg1
 
     def add_file(self, a):
-        self.glist.add_file(a)
+        self.held_object.add_file(a)
 
 class Build(InterpreterObject):
     def __init__(self):
@@ -271,24 +286,28 @@ class Host(InterpreterObject):
 class IncludeDirsHolder(InterpreterObject):
     def __init__(self, curdir, dirs, kwargs):
         super().__init__()
-        self.includedirs = build.IncludeDirs(curdir, dirs, kwargs)
+        self.held_object = build.IncludeDirs(curdir, dirs, kwargs)
 
 class Headers(InterpreterObject):
 
-    def __init__(self, sources, kwargs):
+    def __init__(self, src_subdir, sources, kwargs):
         InterpreterObject.__init__(self)
         self.sources = sources
-        self.subdir = kwargs.get('subdir', '')
+        self.source_subdir = src_subdir
+        self.install_subdir = kwargs.get('subdir', '')
         self.custom_install_dir = kwargs.get('install_dir', None)
         if self.custom_install_dir is not None:
             if not isinstance(self.custom_install_dir, str):
                 raise InterpreterException('Custom_install_dir must be a string.')
 
-    def set_subdir(self, subdir):
-        self.subdir = subdir
+    def set_install_subdir(self, subdir):
+        self.install_subdir = subdir
 
-    def get_subdir(self):
-        return self.subdir
+    def get_install_subdir(self):
+        return self.install_subdir
+
+    def get_source_subdir(self):
+        return self.source_subdir
 
     def get_sources(self):
         return self.sources
@@ -297,31 +316,46 @@ class Headers(InterpreterObject):
         return self.custom_install_dir
 
 class Data(InterpreterObject):
-    def __init__(self, subdir, sources, kwargs):
+    def __init__(self, in_sourcetree, source_subdir, sources, kwargs):
         InterpreterObject.__init__(self)
-        self.subdir = subdir
+        self.in_sourcetree = in_sourcetree
+        self.source_subdir = source_subdir
         self.sources = sources
         kwsource = kwargs.get('sources', [])
         if not isinstance(kwsource, list):
             kwsource = [kwsource]
         self.sources += kwsource
-        self.custom_install_dir = kwargs.get('install_dir', None)
-        if self.custom_install_dir is not None and not isinstance(self.custom_install_dir, str):
+        for s in self.sources:
+            if not isinstance(s, str):
+                raise InterpreterException('Install file name must be a string.')
+        self.install_dir = kwargs.get('install_dir', None)
+        if not isinstance(self.install_dir, str):
             raise InterpreterException('Custom_install_dir must be a string.')
 
-    def get_subdir(self):
-        return self.subdir
+    def get_install_subdir(self):
+        return self.install_subdir
+
+    def get_source_subdir(self):
+        return self.source_subdir
 
     def get_sources(self):
         return self.sources
 
-    def get_custom_install_dir(self):
-        return self.custom_install_dir
+    def get_install_dir(self):
+        return self.install_dir
+
+class InstallDir(InterpreterObject):
+    def __init__(self, source_subdir, installable_subdir, install_dir):
+        InterpreterObject.__init__(self)
+        self.source_subdir = source_subdir
+        self.installable_subdir = installable_subdir
+        self.install_dir = install_dir
 
 class Man(InterpreterObject):
 
-    def __init__(self, sources, kwargs):
+    def __init__(self, source_subdir, sources, kwargs):
         InterpreterObject.__init__(self)
+        self.source_subdir = source_subdir
         self.sources = sources
         self.validate_sources()
         if len(kwargs) > 1:
@@ -342,15 +376,18 @@ class Man(InterpreterObject):
     def get_sources(self):
         return self.sources
 
+    def get_source_subdir(self):
+        return self.source_subdir
+
 class GeneratedObjectsHolder(InterpreterObject):
     def __init__(self, held_object):
         super().__init__()
         self.held_object = held_object
 
 class BuildTargetHolder(InterpreterObject):
-    def __init__(self, targetttype, name, subdir, is_cross, sources, objects, environment, kwargs):
+    def __init__(self, target):
         super().__init__()
-        self.held_object = targetttype(name, subdir, is_cross, sources, objects, environment, kwargs)
+        self.held_object = target
         self.methods.update({'extract_objects' : self.extract_objects_method})
 
     def is_cross(self):
@@ -361,24 +398,24 @@ class BuildTargetHolder(InterpreterObject):
         return GeneratedObjectsHolder(gobjs)
 
 class ExecutableHolder(BuildTargetHolder):
-    def __init__(self, name, subdir, is_cross, sources, objects, environment, kwargs):
-        super().__init__(build.Executable, name, subdir, is_cross, sources, objects, environment, kwargs)
+    def __init__(self, target):
+        super().__init__(target)
 
 class StaticLibraryHolder(BuildTargetHolder):
-    def __init__(self, name, subdir, is_cross, sources, objects, environment, kwargs):
-        super().__init__(build.StaticLibrary, name, subdir, is_cross, sources, objects, environment, kwargs)
+    def __init__(self, target):
+        super().__init__(target)
 
 class SharedLibraryHolder(BuildTargetHolder):
-    def __init__(self, name, subdir, is_cross, sources, objects, environment, kwargs):
-        super().__init__(build.SharedLibrary, name, subdir, is_cross, sources, objects, environment, kwargs)
+    def __init__(self, target):
+        super().__init__(target)
 
 class JarHolder(BuildTargetHolder):
-    def __init__(self, name, subdir, is_cross, sources, objects, environment, kwargs):
-        super().__init__(build.Jar, name, subdir, is_cross, sources, objects, environment, kwargs)
+    def __init__(self, target):
+        super().__init__(target)
 
 class CustomTargetHolder(InterpreterObject):
-    def __init__(self, name, subdir, kwargs):
-        self.held_object = build.CustomTarget(name, subdir, kwargs)
+    def __init__(self, object_to_hold):
+        self.held_object = object_to_hold
 
     def is_cross(self):
         return self.held_object.is_cross()
@@ -392,13 +429,14 @@ class RunTargetHolder(InterpreterObject):
         self.held_object = build.RunTarget(name, command, args, subdir)
 
 class Test(InterpreterObject):
-    def __init__(self, name, exe, is_parallel, cmd_args, env, valgrind_args):
+    def __init__(self, name, exe, is_parallel, cmd_args, env, should_fail, valgrind_args):
         InterpreterObject.__init__(self)
         self.name = name
         self.exe = exe
         self.is_parallel = is_parallel
         self.cmd_args = cmd_args
         self.env = env
+        self.should_fail = should_fail
         self.valgrind_args = valgrind_args
 
     def get_exe(self):
@@ -559,6 +597,36 @@ class CompilerHolder(InterpreterObject):
         mlog.log('Has header "%s":' % string, h)
         return haz
 
+class ModuleState:
+    pass
+
+class ModuleHolder(InterpreterObject):
+    def __init__(self, modname, module, interpreter):
+        InterpreterObject.__init__(self)
+        self.modname = modname
+        self.held_object = module
+        self.interpreter = interpreter
+
+    def method_call(self, method_name, args, kwargs):
+        try:
+            fn = getattr(self.held_object, method_name)
+        except AttributeError:
+            raise InvalidArguments('Module %s does not have method %s.' % (self.modname, method_name))
+        state = ModuleState()
+        state.build_to_src = os.path.relpath(self.interpreter.environment.get_source_dir(),
+                                            self.interpreter.environment.get_build_dir())
+        state.subdir = self.interpreter.subdir
+        state.environment = self.interpreter.environment
+        state.project_name = self.interpreter.build.project_name
+        state.compilers = self.interpreter.build.compilers
+        state.targets = self.interpreter.build.targets
+        state.headers = self.interpreter.build.get_headers()
+        state.man = self.interpreter.build.get_man()
+        state.pkgconfig_gens = self.interpreter.build.pkgconfig_gens
+        state.global_args = self.interpreter.build.global_args
+        value = fn(state, args, kwargs)
+        return self.interpreter.module_method_callback(value)
+
 class MesonMain(InterpreterObject):
     def __init__(self, build, interpreter):
         InterpreterObject.__init__(self)
@@ -690,12 +758,10 @@ class Interpreter():
                       'generator' : self.func_generator,
                       'test' : self.func_test,
                       'install_headers' : self.func_install_headers,
-                      'headers' : self.func_headers,
                       'install_man' : self.func_install_man,
-                      'man' : self.func_man,
                       'subdir' : self.func_subdir,
                       'install_data' : self.func_install_data,
-                      'data' : self.func_data,
+                      'install_subdir' : self.func_install_subdir,
                       'configure_file' : self.func_configure_file,
                       'include_directories' : self.func_include_directories,
                       'add_global_arguments' : self.func_add_global_arguments,
@@ -708,8 +774,43 @@ class Interpreter():
                       'get_option' : self.func_get_option,
                       'subproject' : self.func_subproject,
                       'pkgconfig_gen' : self.func_pkgconfig_gen,
+                      'vcs_tag' : self.func_vcs_tag,
+                      'set_variable' : self.func_set_variable,
+                      'import' : self.func_import,
                       }
 
+    def module_method_callback(self, invalues):
+        unwrap_single = False
+        if invalues is None:
+            return
+        if not isinstance(invalues, list):
+            unwrap_single = True
+            invalues = [invalues]
+        outvalues = []
+        for v in invalues:
+            if isinstance(v, build.CustomTarget):
+                if v.name in self.build.targets:
+                    raise InterpreterException('Tried to create target %s which already exists.' % v.name)
+                self.build.targets[v.name] = v
+                outvalues.append(CustomTargetHolder(v))
+            elif isinstance(v, int) or isinstance(v, str):
+                outvalues.append(v)
+            elif isinstance(v, build.Executable):
+                if v.name in self.build.targets:
+                    raise InterpreterException('Tried to create target %s which already exists.' % v.name)
+                self.build.targets[v.name] = v
+                outvalues.append(ExecutableHolder(v))
+            elif isinstance(v, list):
+                outvalues.append(self.module_method_callback(v))
+            elif isinstance(v, build.GeneratedList):
+                outvalues.append(GeneratedListHolder(v))
+            else:
+                print(v)
+                raise InterpreterException('Module returned a value of unknown type.')
+        if len(outvalues) == 1 and unwrap_single:
+            return outvalues[0]
+        return outvalues
+
     def get_build_def_files(self):
         return self.build_def_files
 
@@ -758,7 +859,33 @@ class Interpreter():
             return self.variables[varname]
         raise InvalidCode('Unknown variable "%s".' % varname)
 
+    def func_set_variable(self, node, args, kwargs):
+        if len(args) != 2:
+            raise InvalidCode('Set_variable takes two arguments.')
+        varname = args[0]
+        value = self.to_native(args[1])
+        self.set_variable(varname, value)
+
+    def func_import(self, node, args, kwargs):
+        if len(args) != 1:
+            raise InvalidCode('Import takes one argument.')
+        modname = args[0]
+        if not isinstance(modname, str):
+            raise InvalidCode('Argument to import was not a string')
+        if not modname in self.environment.coredata.modules:
+            module = importlib.import_module('modules.' + modname).initialize()
+            self.environment.coredata.modules[modname] = module
+        return ModuleHolder(modname, self.environment.coredata.modules[modname], self)
+
     def set_variable(self, varname, variable):
+        if variable is None:
+            raise InvalidCode('Can not assign None to variable.')
+        if not isinstance(varname, str):
+            raise InvalidCode('First argument to set_variable must be a string.')
+        if not self.is_assignable(variable):
+            raise InvalidCode('Assigned value not of assignable type.')
+        if re.fullmatch('[_a-zA-Z][_0-9a-zA-Z]*', varname) is None:
+            raise InvalidCode('Invalid variable name: ' + varname)
         if varname in self.builtin:
             raise InvalidCode('Tried to overwrite internal variable "%s"' % varname)
         self.variables[varname] = variable
@@ -783,13 +910,19 @@ class Interpreter():
         elif isinstance(cur, mparser.ArrayNode):
             return self.evaluate_arraystatement(cur)
         elif isinstance(cur, mparser.NumberNode):
-            return cur
+            return cur.value
         elif isinstance(cur, mparser.AndNode):
             return self.evaluate_andstatement(cur)
         elif isinstance(cur, mparser.OrNode):
             return self.evaluate_orstatement(cur)
         elif isinstance(cur, mparser.NotNode):
             return self.evaluate_notstatement(cur)
+        elif isinstance(cur, mparser.ArithmeticNode):
+            return self.evaluate_arithmeticstatement(cur)
+        elif isinstance(cur, mparser.ForeachClauseNode):
+            return self.evaluate_foreach(cur)
+        elif self.is_elementary_type(cur):
+            return cur
         else:
             raise InvalidCode("Unknown statement.")
 
@@ -890,9 +1023,12 @@ class Interpreter():
         if dirname == self.subprojects:
             return self.subprojects[dirname]
         subdir = os.path.join('subprojects', dirname)
-        abs_subdir = os.path.join(self.build.environment.get_source_dir(), subdir)
-        if not os.path.isdir(abs_subdir):
-            raise InterpreterException('Subproject directory does not exist.')
+        r = wrap.Resolver(os.path.join(self.build.environment.get_source_dir(), 'subprojects'))
+        resolved = r.resolve(dirname)
+        if resolved is None:
+            raise InterpreterException('Subproject directory does not exist and can not be downloaded.')
+        subdir = os.path.join('subprojects', resolved)
+        os.makedirs(os.path.join(self.build.environment.get_build_dir(), subdir), exist_ok=True)
         self.global_args_frozen = True
         mlog.log('\nExecuting subproject ', mlog.bold(dirname), '.\n', sep='')
         subi = Interpreter(self.build, dirname, subdir)
@@ -912,7 +1048,11 @@ class Interpreter():
         if not isinstance(optname, str):
             raise InterpreterException('Argument of get_option must be a string.')
         if self.subproject != '':
-            optname = self.subproject + '-' + optname
+            optname = self.subproject + ':' + optname
+        try:
+            return self.environment.get_coredata().get_builtin_option(optname)
+        except RuntimeError:
+            pass
         if optname not in self.environment.coredata.user_options:
             raise InterpreterException('Tried to access unknown option "%s".' % optname)
         return self.environment.coredata.user_options[optname].value
@@ -1057,7 +1197,7 @@ class Interpreter():
         if not dep.found():
             dep = dependencies.find_external_dependency(name, kwargs)
         self.coredata.deps[identifier] = dep
-        return dep
+        return DependencyHolder(dep)
 
     def func_executable(self, node, args, kwargs):
         return self.build_target(node, args, kwargs, ExecutableHolder)
@@ -1071,6 +1211,33 @@ class Interpreter():
     def func_jar(self, node, args, kwargs):
         return self.build_target(node, args, kwargs, JarHolder)
 
+    def func_vcs_tag(self, node, args, kwargs):
+        fallback = kwargs.pop('fallback', None)
+        if not isinstance(fallback, str):
+            raise InterpreterException('Keyword argument must exist and be a string.')
+        replace_string = kwargs.pop('replace_string', '@VCS_TAG@')
+        regex_selector = '(.*)' # default regex selector for custom command: use complete output
+        vcs_cmd = kwargs.get('command', None)
+        if vcs_cmd and not isinstance(vcs_cmd, list):
+            vcs_cmd = [vcs_cmd]
+        source_dir = os.path.normpath(os.path.join(self.environment.get_source_dir(), self.subdir))
+        if vcs_cmd:
+            # Is the command an executable in path or maybe a script in the source tree?
+            vcs_cmd[0] = shutil.which(vcs_cmd[0]) or os.path.join(source_dir, vcs_cmd[0])
+        else:
+            vcs = mesonlib.detect_vcs(source_dir)
+            if vcs:
+                mlog.log('Found %s repository at %s' % (vcs['name'], vcs['wc_dir']))
+                vcs_cmd = vcs['get_rev'].split()
+                regex_selector = vcs['rev_regex']
+            else:
+                vcs_cmd = [' '] # executing this cmd will fail in vcstagger.py and force to use the fallback string
+        scriptfile = os.path.join(self.environment.get_script_dir(), 'vcstagger.py')
+        # vcstagger.py parameters: infile, outfile, fallback, source_dir, replace_string, regex_selector, command...
+        kwargs['command'] = [sys.executable, scriptfile, '@INPUT0@', '@OUTPUT0@', fallback, source_dir, replace_string, regex_selector] + vcs_cmd
+        kwargs.setdefault('build_always', True)
+        return self.func_custom_target(node, [kwargs['output']], kwargs)
+
     def func_custom_target(self, node, args, kwargs):
         if len(args) != 1:
             raise InterpreterException('Incorrect number of arguments')
@@ -1082,7 +1249,7 @@ class Interpreter():
                                    % name)
         if name in self.build.targets:
             raise InvalidCode('Tried to create target "%s", but a target of that name already exists.' % name)
-        tg = CustomTargetHolder(name, self.subdir, kwargs)
+        tg = CustomTargetHolder(build.CustomTarget(name, self.subdir, kwargs))
         self.build.targets[name] = tg.held_object
         return tg
 
@@ -1114,7 +1281,7 @@ class Interpreter():
             raise InterpreterException('Incorrect number of arguments')
         if not isinstance(args[0], str):
             raise InterpreterException('First argument of test must be a string.')
-        if not isinstance(args[1], ExecutableHolder) and not isinstance(args[1], JarHolder):
+        if not isinstance(args[1], (ExecutableHolder, JarHolder, ExternalProgramHolder)):
             raise InterpreterException('Second argument must be executable.')
         par = kwargs.get('is_parallel', True)
         if not isinstance(par, bool):
@@ -1144,32 +1311,27 @@ class Interpreter():
         for a in valgrind_args:
             if not isinstance(a, str):
                 raise InterpreterException('Valgrind_arg not a string.')
-        t = Test(args[0], args[1].held_object, par, cmd_args, env, valgrind_args)
+        should_fail = kwargs.get('should_fail', False)
+        if not isinstance(should_fail, bool):
+            raise InterpreterException('Keyword argument should_fail must be a boolean.')
+        t = Test(args[0], args[1].held_object, par, cmd_args, env, should_fail, valgrind_args)
         self.build.tests.append(t)
         mlog.debug('Adding test "', mlog.bold(args[0]), '".', sep='')
 
-    def func_headers(self, node, args, kwargs):
-        mlog.log(mlog.bold('Warning! '), 'Function headers() is deprecated, please use install_headers() instead.')
-        return self.func_install_headers(node, args, kwargs)
-
     def func_install_headers(self, node, args, kwargs):
         args = self.flatten(args)
         for a in args:
             if not isinstance(a, str):
                 raise InvalidArguments('Argument %s is not a string.' % str(a))
-        h = Headers(args, kwargs)
+        h = Headers(self.subdir, args, kwargs)
         self.build.headers.append(h)
         return h
 
-    def func_man(self, node, args, kwargs):
-        mlog.log(mlog.bold('Warning! '), 'Function man() is deprecated, please use install_man() instead.')
-        return self.func_install_man(node, args, kwargs)
-
     def func_install_man(self, node, args, kwargs):
         for a in args:
             if not isinstance(a, str):
                 raise InvalidArguments('Argument %s is not a string.' % str(a))
-        m = Man(args, kwargs)
+        m = Man(self.subdir, args, kwargs)
         self.build.man.append(m)
         return m
 
@@ -1207,20 +1369,29 @@ class Interpreter():
         self.evaluate_codeblock(codeblock)
         self.subdir = prev_subdir
 
-    def func_data(self, node, args, kwargs):
-        mlog.log(mlog.bold('Warning! '), 'Function data() is deprecated, please use install_data() instead.')
-        return self.func_install_data(node, args, kwargs)
-
     def func_install_data(self, node, args, kwargs):
-        if len(args ) < 1:
-            raise InvalidArguments('Data function must have at least one argument: the subdirectory.')
         for a in args:
             if not isinstance(a, str):
                 raise InvalidArguments('Argument %s is not a string.' % str(a))
-        data = Data(args[0], args[1:], kwargs)
+        data = Data(True, self.subdir, args, kwargs)
         self.build.data.append(data)
         return data
 
+    def func_install_subdir(self, node, args, kwargs):
+        if len(args ) != 1:
+            raise InvalidArguments('Install_subdir requires exactly one argument.')
+        for a in args:
+            if not isinstance(a, str):
+                raise InvalidArguments('Argument %s is not a string.' % str(a))
+        if not 'install_dir' in kwargs:
+            raise InvalidArguments('Missing keyword argument install_dir')
+        install_dir = kwargs['install_dir']
+        if not isinstance(install_dir, str):
+            raise InvalidArguments('Keyword argument install_dir not a string.')
+        idir = InstallDir(self.subdir, args[0], install_dir)
+        self.build.install_dirs.append(idir)
+        return idir
+
     def func_configure_file(self, node, args, kwargs):
         if len(args) > 0:
             raise InterpreterException("configure_file takes only keyword arguments.")
@@ -1228,17 +1399,23 @@ class Interpreter():
             raise InterpreterException('Required keyword argument "input" not defined.')
         if not 'output' in kwargs:
             raise InterpreterException('Required keyword argument "output" not defined.')
+        inputfile = kwargs['input']
+        output = kwargs['output']
+        if not isinstance(inputfile, str):
+            raise InterpreterException('Input must be a string.')
+        if not isinstance(output, str):
+            raise InterpreterException('Output must be a string.')
         if 'configuration' in kwargs:
-            inputfile = kwargs['input']
-            output = kwargs['output']
             conf = kwargs['configuration']
             if not isinstance(conf, ConfigurationDataHolder):
                 raise InterpreterException('Argument "configuration" is not of type configuration_data')
 
             conffile = os.path.join(self.subdir, inputfile)
             self.build_def_files.append(conffile)
-            c = ConfigureFileHolder(self.subdir, inputfile, output, conf.held_object)
-            self.build.configure_files.append(c.held_object)
+            os.makedirs(os.path.join(self.environment.build_dir, self.subdir), exist_ok=True)
+            ifile_abs = os.path.join(self.environment.source_dir, self.subdir, inputfile)
+            ofile_abs = os.path.join(self.environment.build_dir, self.subdir, output)
+            mesonlib.do_conf_file(ifile_abs, ofile_abs, conf.held_object)
             conf.mark_used()
         elif 'command' in kwargs:
             res = self.func_run_command(node, kwargs['command'], {})
@@ -1247,6 +1424,14 @@ class Interpreter():
                                            (res.stdout, res.stderr))
         else:
             raise InterpreterException('Configure_file must have either "configuration" or "command".')
+        if isinstance(kwargs.get('install_dir', None), str):
+            self.build.data.append(Data(False, self.subdir, [output], kwargs))
+        # FIXME, HORROR, this is a massive hack to get this working. The correct
+        # solution is to finally do the refactoring where source files are no
+        # longer strings but actual objects. This is a major undertaking
+        # and will only be done after the next release.
+        outputfile = os.path.join(self.environment.build_dir, self.subdir, output)
+        return outputfile
 
     def func_include_directories(self, node, args, kwargs):
         absbase = os.path.join(self.environment.get_source_dir(), self.subdir)
@@ -1266,7 +1451,7 @@ class Interpreter():
         if self.subproject != '':
             raise InvalidCode('Global arguments can not be set in subprojects because there is no way to make that reliable.')
         if self.global_args_frozen:
-            raise InvalidCode('Tried to set global arguments after they have become immutable.')
+            raise InvalidCode('Tried to set global arguments after a build target has been declared.\nThis is not permitted. Please declare all global arguments before your targets.')
         if not 'language' in kwargs:
             raise InvalidCode('Missing language definition in add_global_arguments')
         lang = kwargs['language'].lower()
@@ -1282,6 +1467,8 @@ class Interpreter():
             return args
         if isinstance(args, InterpreterObject):
             return args
+        if isinstance(args, int):
+            return args
         result = []
         for a in args:
             if isinstance(a, list):
@@ -1293,7 +1480,7 @@ class Interpreter():
                 result.append(a)
         return result
 
-    def build_target(self, node, args, kwargs, targetclass):
+    def build_target(self, node, args, kwargs, targetholder):
         args = self.flatten(args)
         name = args[0]
         sources = args[1:]
@@ -1320,15 +1507,22 @@ class Interpreter():
         if name in self.build.targets:
             raise InvalidCode('Tried to create target "%s", but a target of that name already exists.' % name)
         self.check_sources_exist(os.path.join(self.source_root, self.subdir), sources)
-        l = targetclass(name, self.subdir, is_cross, sources, objs, self.environment, kwargs)
+        if targetholder is ExecutableHolder:
+            targetclass = build.Executable
+        elif targetholder is SharedLibraryHolder:
+            targetclass = build.SharedLibrary
+        elif targetholder is StaticLibraryHolder:
+            targetclass = build.StaticLibrary
+        elif targetholder is JarHolder:
+            targetclass = build.Jar
+        else:
+            print(targetholder)
+            raise RuntimeError('Unreachable code')
+        target = targetclass(name, self.subdir, is_cross, sources, objs, self.environment, kwargs)
+        l = targetholder(target)
         self.build.targets[name] = l.held_object
         if name not in self.coredata.target_guids:
             self.coredata.target_guids[name] = str(uuid.uuid4()).upper()
-        if self.environment.is_cross_build() and l.is_cross:
-            txt = ' cross build '
-        else:
-            txt = ' build '
-        displayname = os.path.join(l.held_object.subdir, name)
         self.global_args_frozen = True
         return l
 
@@ -1363,45 +1557,23 @@ class Interpreter():
         if not isinstance(var_name, str):
             raise InvalidArguments('Tried to assign value to a non-variable.')
         value = self.evaluate_statement(node.value)
-        if value is None:
-            raise InvalidCode('Can not assign None to variable.')
         value = self.to_native(value)
         if not self.is_assignable(value):
             raise InvalidCode('Tried to assign an invalid value to variable.')
         self.set_variable(var_name, value)
         return value
 
-    def reduce_single(self, arg):
-        if isinstance(arg, mparser.IdNode):
-            return self.get_variable(arg.value)
-        elif isinstance(arg, str):
-            return arg
-        elif isinstance(arg, mparser.StringNode):
-            return arg.value
-        elif isinstance(arg, mparser.FunctionNode):
-            return self.function_call(arg)
-        elif isinstance(arg, mparser.MethodNode):
-            return self.method_call(arg)
-        elif isinstance(arg, mparser.BooleanNode):
-            return arg.value
-        elif isinstance(arg, mparser.ArrayNode):
-            return [self.reduce_single(curarg) for curarg in arg.args.arguments]
-        elif isinstance(arg, mparser.NumberNode):
-            return arg.value
-        else:
-            raise InvalidCode('Irreducible argument.')
-
     def reduce_arguments(self, args):
         assert(isinstance(args, mparser.ArgumentNode))
         if args.incorrect_order():
             raise InvalidArguments('All keyword arguments must be after positional arguments.')
-        reduced_pos = [self.reduce_single(arg) for arg in args.arguments]
+        reduced_pos = [self.evaluate_statement(arg) for arg in args.arguments]
         reduced_kw = {}
         for key in args.kwargs.keys():
             if not isinstance(key, str):
                 raise InvalidArguments('Keyword argument name is not a string.')
             a = args.kwargs[key]
-            reduced_kw[key] = self.reduce_single(a)
+            reduced_kw[key] = self.evaluate_statement(a)
         return (reduced_pos, reduced_kw)
 
     def string_method_call(self, obj, method_name, args):
@@ -1435,7 +1607,7 @@ class Interpreter():
         if isinstance(args, mparser.ArgumentNode):
             args = args.arguments
         for (i, arg) in enumerate(args):
-            arg = self.to_native(self.reduce_single(arg))
+            arg = self.to_native(self.evaluate_statement(arg))
             if isinstance(arg, bool): # Python boolean is upper case.
                 arg = str(arg).lower()
             templ = templ.replace('@{}@'.format(i), str(arg))
@@ -1466,6 +1638,13 @@ class Interpreter():
     def array_method_call(self, obj, method_name, args):
         if method_name == 'contains':
             return self.check_contains(obj, args)
+        elif method_name == 'get':
+            index = args[0]
+            if not isinstance(index, int):
+                raise InvalidArguments('Array index must be a number.')
+            if index < 0 or index >= len(obj):
+                raise InvalidArguments('Array index %s is out of bounds for array of size %d.' % (index, len(obj)))
+            return obj[index]
         raise InterpreterException('Arrays do not have a method called "%s".' % method_name)
 
     def check_contains(self, obj, args):
@@ -1497,8 +1676,18 @@ class Interpreter():
         if not isinstance(node.elseblock, mparser.EmptyNode):
             self.evaluate_codeblock(node.elseblock)
 
+    def evaluate_foreach(self, node):
+        assert(isinstance(node, mparser.ForeachClauseNode))
+        varname = node.varname.value
+        items = self.evaluate_statement(node.items)
+        if not isinstance(items, list):
+            raise InvalidArguments('Items of foreach loop is not an array')
+        for item in items:
+            self.set_variable(varname, item)
+            self.evaluate_codeblock(node.block)
+
     def is_elementary_type(self, v):
-        if isinstance(v, int) or isinstance(v, str) or isinstance(v, bool):
+        if isinstance(v, (int, float, str, bool, list)):
             return True
         return False
 
@@ -1513,9 +1702,6 @@ class Interpreter():
             val2 = v2
         else:
             val2 = v2.value
-        if type(val1) != type(val2):
-            raise InterpreterException('Comparison of different types %s and %s.' %
-                                       (str(type(val1)), str(type(val2))))
         if node.ctype == '==':
             return val1 == val2
         elif node.ctype == '!=':
@@ -1561,6 +1747,30 @@ class Interpreter():
             raise InterpreterException('Argument to "not" is not a boolean.')
         return not v
 
+    def evaluate_arithmeticstatement(self, cur):
+        l = self.to_native(self.evaluate_statement(cur.left))
+        r = self.to_native(self.evaluate_statement(cur.right))
+
+        if cur.operation == 'add':
+            try:
+                return l + r
+            except Exception as e:
+                raise InvalidCode('Invalid use of addition: ' + str(e))
+        elif cur.operation == 'sub':
+            if not isinstance(l, int) or not isinstance(r, int):
+                raise InvalidCode('Subtraction works only with integers.')
+            return l - r
+        elif cur.operation == 'mul':
+            if not isinstance(l, int) or not isinstance(r, int):
+                raise InvalidCode('Multiplication works only with integers.')
+            return l * r
+        elif cur.operation == 'div':
+            if not isinstance(l, int) or not isinstance(r, int):
+                raise InvalidCode('Division works only with integers.')
+            return l // r
+        else:
+            raise InvalidCode('You broke me.')
+
     def evaluate_arraystatement(self, cur):
         (arguments, kwargs) = self.reduce_arguments(cur.args)
         if len(kwargs) > 0:
