@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 
-# Copyright 2013-2014 Jussi Pakkanen
+# Copyright 2013-2014 The Meson development team
 
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -30,17 +30,19 @@ class InstallData():
         self.po_package_name = ''
         self.po = []
         self.install_script = None
+        self.install_subdirs = []
 
 def do_install(datafilename):
     ifile = open(datafilename, 'rb')
     d = pickle.load(ifile)
     destdir_var = 'DESTDIR'
     if destdir_var in os.environ:
-        if d.prefix[0] == '/':
-            subdir = d.prefix[1:]
-        else:
-            subdir = d.prefix
-        d.prefix = os.path.join(os.environ[destdir_var], subdir)
+        d.destdir = os.environ[destdir_var]
+    else:
+        d.destdir = ''
+    d.fullprefix = d.destdir + d.prefix
+
+    install_subdirs(d) # Must be first, because it needs to delete the old subtree.
     install_targets(d)
     install_headers(d)
     install_man(d)
@@ -48,13 +50,28 @@ def do_install(datafilename):
     install_po(d)
     run_install_script(d)
 
+def install_subdirs(d):
+    for (src_dir, dst_dir) in d.install_subdirs:
+        if os.path.isabs(dst_dir):
+            dst_dir = d.destdir + dst_dir
+        else:
+            dst_dir = d.fullprefix + dst_dir
+        # Python's copytree works in strange ways.
+        last_level = os.path.split(src_dir)[-1]
+        final_dst = os.path.join(dst_dir, last_level)
+# Don't do rmtree because final_dst might point to e.g. /var/www
+# We might need to revert to walking the directory tree by hand.
+#        shutil.rmtree(final_dst, ignore_errors=True)
+        shutil.copytree(src_dir, final_dst, symlinks=True)
+        print('Installing subdir %s to %s.' % (src_dir, dst_dir))
+
 def install_po(d):
     packagename = d.po_package_name
     for f in d.po:
         srcfile = f[0]
         localedir = f[1]
         languagename = f[2]
-        outfile = os.path.join(d.prefix, localedir, languagename, 'LC_MESSAGES',
+        outfile = os.path.join(d.fullprefix, localedir, languagename, 'LC_MESSAGES',
                                packagename + '.mo')
         os.makedirs(os.path.split(outfile)[0], exist_ok=True)
         shutil.copyfile(srcfile, outfile)
@@ -64,9 +81,13 @@ def install_po(d):
 def install_data(d):
     for i in d.data:
         fullfilename = i[0]
-        outfilerel = i[1]
-        outdir = os.path.join(d.prefix, os.path.split(outfilerel)[0])
-        outfilename = os.path.join(outdir, os.path.split(outfilerel)[1])
+        outfilename = i[1]
+        if os.path.isabs(outfilename):
+            outdir = d.destdir + os.path.split(outfilename)[0]
+            outfilename = d.destdir + outfilename
+        else:
+            outdir = os.path.join(d.fullprefix, os.path.split(outfilename)[0])
+            outfilename = os.path.join(outdir, os.path.split(outfilename)[1])
         os.makedirs(outdir, exist_ok=True)
         print('Installing %s to %s.' % (fullfilename, outdir))
         shutil.copyfile(fullfilename, outfilename)
@@ -75,7 +96,7 @@ def install_data(d):
 def install_man(d):
     for m in d.man:
         outfileroot = m[1]
-        outfilename = os.path.join(d.prefix, outfileroot)
+        outfilename = os.path.join(d.fullprefix, outfileroot)
         full_source_filename = m[0]
         outdir = os.path.split(outfilename)[0]
         os.makedirs(outdir, exist_ok=True)
@@ -89,7 +110,7 @@ def install_man(d):
 def install_headers(d):
     for t in d.headers:
         fullfilename = t[0]
-        outdir = os.path.join(d.prefix, t[1])
+        outdir = os.path.join(d.fullprefix, t[1])
         fname = os.path.split(fullfilename)[1]
         outfilename = os.path.join(outdir, fname)
         print('Installing %s to %s' % (fname, outdir))
@@ -153,7 +174,7 @@ def check_for_stampfile(fname):
 def install_targets(d):
     for t in d.targets:
         fname = check_for_stampfile(t[0])
-        outdir = os.path.join(d.prefix, t[1])
+        outdir = os.path.join(d.fullprefix, t[1])
         aliases = t[2]
         outname = os.path.join(outdir, os.path.split(fname)[-1])
         should_strip = t[3]
@@ -179,7 +200,7 @@ def install_targets(d):
                     os.unlink(symlinkfilename)
                 except FileNotFoundError:
                     pass
-                os.symlink(fname, symlinkfilename)
+                os.symlink(os.path.split(fname)[-1], symlinkfilename)
             except NotImplementedError:
                 if not printed_symlink_error:
                     print("Symlink creation does not work on this platform.")
