@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 
-# Copyright 2013-2014 Jussi Pakkanen
+# Copyright 2013-2015 The Meson development team
 
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,16 +16,19 @@
 
 import sys, os, subprocess, time, datetime, pickle, multiprocessing, json
 import concurrent.futures as conc
-from optparse import OptionParser
+import argparse
 import platform
 
 tests_failed = False
 
-parser = OptionParser()
-parser.add_option('--wrapper', default=None, dest='wrapper',
+parser = argparse.ArgumentParser()
+parser.add_argument('--wrapper', default=None, dest='wrapper',
                   help='wrapper to run tests with (e.g. valgrind)')
-parser.add_option('--wd', default=None, dest='wd',
+parser.add_argument('--wd', default=None, dest='wd',
                   help='directory to cd into before running')
+parser.add_argument('args', nargs='+')
+
+
 class TestRun():
     def __init__(self, res, returncode, duration, stdo, stde, cmd):
         self.res = res
@@ -62,10 +65,10 @@ def run_with_mono(fname):
 
 def run_single_test(wrap, test):
     global tests_failed
-    if test.fname.endswith('.jar'):
-        cmd = ['java', '-jar', test.fname]
-    elif run_with_mono(test.fname):
-        cmd = ['mono', test.fname]
+    if test.fname[0].endswith('.jar'):
+        cmd = ['java', '-jar'] + test.fname
+    elif run_with_mono(test.fname[0]):
+        cmd = ['mono'] + test.fname
     else:
         if test.is_cross:
             if test.exe_runner is None:
@@ -73,9 +76,9 @@ def run_single_test(wrap, test):
                 # because there is no execute wrapper.
                 cmd = None
             else:
-                cmd = [test.exe_runner, test.fname]
+                cmd = [test.exe_runner] + test.fname
         else:
-            cmd = [test.fname]
+            cmd = test.fname
     if len(wrap) > 0 and 'valgrind' in wrap[0]:
         wrap += test.valgrind_args
     if cmd is None:
@@ -96,7 +99,8 @@ def run_single_test(wrap, test):
         duration = endtime - starttime
         stdo = stdo.decode()
         stde = stde.decode()
-        if p.returncode == 0:
+        if (not test.should_fail and p.returncode == 0) or \
+            (test.should_fail and p.returncode != 0):
             res = 'OK'
         else:
             res = 'FAIL'
@@ -107,9 +111,9 @@ def run_single_test(wrap, test):
 def print_stats(numlen, tests, name, result, i, logfile, jsonlogfile):
     startpad = ' '*(numlen - len('%d' % (i+1)))
     num = '%s%d/%d' % (startpad, i+1, len(tests))
-    padding1 = ' '*(40-len(name))
+    padding1 = ' '*(38-len(name))
     padding2 = ' '*(5-len(result.res))
-    result_str = '%s %s%s%s%s(%5.2f s)' % \
+    result_str = '%s %s  %s%s%s(%5.2f s)' % \
         (num, name, padding1, result.res, padding2, result.duration)
     print(result_str)
     write_log(logfile, name, result_str, result)
@@ -134,13 +138,16 @@ def run_tests(options, datafilename):
     jsonlogfile = open(jsonlogfilename, 'w')
     logfile.write('Log of Meson test suite run on %s.\n\n' % datetime.datetime.now().isoformat())
     tests = pickle.load(open(datafilename, 'rb'))
+    if len(tests) == 0:
+        print('No tests defined.')
+        return
     numlen = len('%d' % len(tests))
     varname = 'MESON_TESTTHREADS'
     if varname in os.environ:
         try:
             num_workers = int(os.environ[varname])
         except ValueError:
-            write_log('Invalid value in %s, using 1 thread.' % varname)
+            print('Invalid value in %s, using 1 thread.' % varname)
             num_workers = 1
     else:
         num_workers = multiprocessing.cpu_count()
@@ -159,13 +166,13 @@ def run_tests(options, datafilename):
     print('\nFull log written to %s.' % logfilename)
 
 if __name__ == '__main__':
-    (options, args) = parser.parse_args(sys.argv)
-    if len(args) != 2:
+    options = parser.parse_args()
+    if len(options.args) != 1:
         print('Test runner for Meson. Do not run on your own, mmm\'kay?')
         print('%s [data file]' % sys.argv[0])
     if options.wd is not None:
         os.chdir(options.wd)
-    datafile = args[1]
+    datafile = options.args[0]
     run_tests(options, datafile)
     if tests_failed:
         sys.exit(1)
