@@ -1,6 +1,4 @@
-#!/usr/bin/python3
-
-# Copyright 2014 Jussi Pakkanen
+# Copyright 2014-2015 The Meson development team
 
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,7 +13,6 @@
 # limitations under the License.
 
 import re
-import sys
 from coredata import MesonException
 
 class ParseException(MesonException):
@@ -39,7 +36,7 @@ class Token:
 class Lexer:
     def __init__(self):
         self.keywords = {'true', 'false', 'if', 'else', 'elif',
-                         'endif', 'and', 'or', 'not'}
+                         'endif', 'and', 'or', 'not', 'foreach', 'endforeach'}
         self.token_specification = [
             # Need to be sorted longest to shortest.
             ('ignore', re.compile(r'[ \t]')),
@@ -53,9 +50,13 @@ class Lexer:
             ('rparen', re.compile(r'\)')),
             ('lbracket', re.compile(r'\[')),
             ('rbracket', re.compile(r'\]')),
-            ('string', re.compile("'[^']*?'")),
+            ('string', re.compile(r"'([^'\\]|(\\.))*'")),
             ('comma', re.compile(r',')),
             ('dot', re.compile(r'\.')),
+            ('plus', re.compile(r'\+')),
+            ('dash', re.compile(r'-')),
+            ('star', re.compile(r'\*')),
+            ('fslash', re.compile(r'/')),
             ('colon', re.compile(r':')),
             ('equal', re.compile(r'==')),
             ('nequal', re.compile(r'\!=')),
@@ -91,7 +92,8 @@ class Lexer:
                     elif tid == 'rbracket':
                         bracket_count -= 1
                     elif tid == 'string':
-                        value = match_text[1:-1]
+                        value = match_text[1:-1].replace(r"\'", "'").replace(r" \\ ".strip(), r" \ ".strip())\
+                        .replace("\\n", "\n")
                     elif tid == 'multiline_string':
                         tid = 'string'
                         value = match_text[3:-3]
@@ -129,6 +131,9 @@ class IdNode:
         self.value = token.value
         assert(isinstance(self.value, str))
 
+    def __str__(self):
+        return "Id node: '%s' (%d, %d)." % (self.value, self.lineno, self.colno)
+
 class NumberNode:
     def __init__(self, token):
         self.lineno = token.lineno
@@ -143,6 +148,9 @@ class StringNode:
         self.value = token.value
         assert(isinstance(self.value, str))
 
+    def __str__(self):
+        return "String node: '%s' (%d, %d)." % (self.value, self.lineno, self.colno)
+
 class ArrayNode:
     def __init__(self, args):
         self.lineno = args.lineno
@@ -177,6 +185,14 @@ class ComparisonNode:
         self.right = right
         self.ctype = ctype
 
+class ArithmeticNode:
+    def __init__(self, lineno, colno, operation, left, right):
+        self.lineno = lineno
+        self.colno = colno
+        self.left = left
+        self.right = right
+        self.operation = operation
+
 class NotNode:
     def __init__(self, lineno, colno, value):
         self.lineno = lineno
@@ -214,6 +230,14 @@ class AssignmentNode:
         assert(isinstance(var_name, str))
         self.value = value
 
+class ForeachClauseNode():
+    def __init__(self, lineno, colno, varname, items, block):
+        self.lineno = lineno
+        self.colno = colno
+        self.varname = varname
+        self.items = items
+        self.block = block
+
 class IfClauseNode():
     def __init__(self, lineno, colno):
         self.lineno = lineno
@@ -237,16 +261,18 @@ class ArgumentNode():
         self.order_error = False
 
     def prepend(self, statement):
+        if self.num_kwargs() > 0:
+            self.order_error = True
         if not isinstance(statement, EmptyNode):
             self.arguments = [statement] + self.arguments
 
     def append(self, statement):
+        if self.num_kwargs() > 0:
+            self.order_error = True
         if not isinstance(statement, EmptyNode):
             self.arguments = self.arguments + [statement]
 
     def set_kwarg(self, name, value):
-        if self.num_args() > 0:
-            self.order_error = True
         self.kwargs[name] = value
 
     def num_args(self):
@@ -269,12 +295,12 @@ class ArgumentNode():
 # 1 assignment
 # 2 or
 # 3 and
-# 4 equality
-# comparison, plus and multiplication would go here
-# 5 negation
-# 6 funcall, method call
-# 7 parentheses
-# 8 plain token
+# 4 comparison
+# 5 arithmetic
+# 6 negation
+# 7 funcall, method call
+# 8 parentheses
+# 9 plain token
 
 class Parser:
     def __init__(self, code):
@@ -337,12 +363,39 @@ class Parser:
         return left
 
     def e5(self):
-        if self.accept('not'):
-            return NotNode(self.current.lineno, self.current.colno, self.e6())
-        return self.e6()
+        return self.e5add()
+
+    def e5add(self):
+        left = self.e5sub()
+        if self.accept('plus'):
+            return ArithmeticNode(left.lineno, left.colno, 'add', left, self.e5add())
+        return left
+
+    def e5sub(self):
+        left = self.e5mul()
+        if self.accept('dash'):
+            return ArithmeticNode(left.lineno, left.colno, 'sub', left, self.e5sub())
+        return left
+
+    def e5mul(self):
+        left = self.e5div()
+        if self.accept('star'):
+            return ArithmeticNode(left.lineno, left.colno, 'mul', left, self.e5mul())
+        return left
+
+    def e5div(self):
+        left = self.e6()
+        if self.accept('fslash'):
+            return ArithmeticNode(left.lineno, left.colno, 'div', left, self.e5div())
+        return left
 
     def e6(self):
-        left = self.e7()
+        if self.accept('not'):
+            return NotNode(self.current.lineno, self.current.colno, self.e7())
+        return self.e7()
+
+    def e7(self):
+        left = self.e8()
         if self.accept('lparen'):
             args = self.args()
             self.expect('rparen')
@@ -354,7 +407,7 @@ class Parser:
             left = self.method_call(left)
         return left
 
-    def e7(self):
+    def e8(self):
         if self.accept('lparen'):
             e = self.statement()
             self.expect('rparen')
@@ -364,9 +417,9 @@ class Parser:
             self.expect('rbracket')
             return ArrayNode(args)
         else:
-            return self.e8()
+            return self.e9()
 
-    def e8(self):
+    def e9(self):
         t = self.current
         if self.accept('true'):
             return BooleanNode(t, True);
@@ -382,30 +435,26 @@ class Parser:
 
     def args(self):
         s = self.statement()
-        if isinstance(s, EmptyNode):
-            ArgumentNode(s)
+        a = ArgumentNode(s)
 
-        if self.accept('comma'):
-            rest = self.args()
-            rest.prepend(s)
-            return rest
-        if self.accept('colon'):
-            if not isinstance(s, IdNode):
-                raise ParseException('Keyword argument must be a plain identifier.',
-                                     s.lineno, s.colno)
-            value = self.statement()
+        while not isinstance(s, EmptyNode):
             if self.accept('comma'):
-                a = self.args()
+                a.append(s)
+            elif self.accept('colon'):
+                if not isinstance(s, IdNode):
+                    raise ParseException('Keyword argument must be a plain identifier.',
+                                         s.lineno, s.colno)
+                a.set_kwarg(s.value, self.statement())
+                if not self.accept('comma'):
+                    return a
             else:
-                a = ArgumentNode(self.current)
-            a.set_kwarg(s.value, value)
-            return a
-        a = ArgumentNode(self.current)
-        a.append(s)
+                a.append(s)
+                return a
+            s = self.statement()
         return a
 
     def method_call(self, source_object):
-        methodname = self.e8()
+        methodname = self.e9()
         if not(isinstance(methodname, IdNode)):
             raise ParseException('Method name must be plain id',
                                  self.current.lineno, self.current.colno)
@@ -417,6 +466,15 @@ class Parser:
             return self.method_call(method)
         return method
 
+    def foreachblock(self):
+        t = self.current
+        self.expect('id')
+        varname = t
+        self.expect('colon')
+        items = self.statement()
+        block = self.codeblock()
+        return ForeachClauseNode(varname.lineno, varname.colno, varname, items, block)
+
     def ifblock(self):
         condition = self.statement()
         clause = IfClauseNode(condition.lineno, condition.colno)
@@ -445,6 +503,10 @@ class Parser:
             block = self.ifblock()
             self.expect('endif')
             return block
+        if self.accept('foreach'):
+            block = self.foreachblock()
+            self.expect('endforeach')
+            return block
         return self.statement()
 
     def codeblock(self):
@@ -456,13 +518,3 @@ class Parser:
                 block.lines.append(curline)
             cond = self.accept('eol')
         return block
-
-if __name__ == '__main__':
-    for code in sys.argv[1:]:
-        parser = Parser(open(code).read())
-        try:
-            print(code)
-            parser.parse()
-        except ParseException as e:
-            print('Error', e.text, 'line', e.lineno, 'column', e.colno)
-
