@@ -1,4 +1,4 @@
-# Copyright 2012-2014 Jussi Pakkanen
+# Copyright 2012-2014 The Meson development team
 
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,7 +13,7 @@
 # limitations under the License.
 
 import backends
-import environment
+import environment, mesonlib
 import build
 import mlog
 import dependencies
@@ -22,7 +22,7 @@ from build import InvalidArguments
 from coredata import MesonException
 import os, sys, shutil, pickle, re
 
-if environment.is_windows():
+if mesonlib.is_windows():
     quote_char = '"'
     execute_wrapper = 'cmd /c'
 else:
@@ -91,7 +91,7 @@ class NinjaBuildElement():
         for e in self.elems:
             (name, elems) = e
             should_quote = True
-            if name == 'DEPFILE' or name == 'DESC':
+            if name == 'DEPFILE' or name == 'DESC' or name == 'pool':
                 should_quote = False
             line = ' %s = ' % name
             q_templ = quote_char + "%s" + quote_char
@@ -120,13 +120,13 @@ class NinjaBackend(backends.Backend):
         outfilename = os.path.join(self.environment.get_build_dir(), self.ninja_filename)
         tempfilename = outfilename + '~'
         outfile = open(tempfilename, 'w')
-        self.generate_configure_files()
         self.generate_pkgconfig_files()
         outfile.write('# This is the build file for project "%s"\n' % self.build.get_project())
         outfile.write('# It is autogenerated by the Meson build system.\n')
         outfile.write('# Do not edit by hand.\n\n')
-        outfile.write('ninja_required_version = 1.3.4\n\n')
+        outfile.write('ninja_required_version = 1.5.1\n\n')
         self.generate_rules(outfile)
+        self.generate_phony(outfile)
         outfile.write('# Build rules for targets\n\n')
         [self.generate_target(t, outfile) for t in self.build.get_targets().values()]
         if len(self.build.pot) > 0:
@@ -206,6 +206,8 @@ class NinjaBackend(backends.Backend):
                         else:
                             obj_list.append(self.generate_single_compile(target, outfile, RawFilename(src), True,
                                                                          header_deps))
+                    elif self.environment.is_object(src):
+                        obj_list.append(src)
                     else:
                         # Assume anything not specifically a source file is a header. This is because
                         # people generate files with weird suffixes (.inc, .fh) that they then include
@@ -278,6 +280,8 @@ class NinjaBackend(backends.Backend):
         deps = [os.path.join(i.get_subdir(), self.hackety_hack(i.get_filename())) for i in target.get_dependencies()]
         srcs = [os.path.join(self.build_to_src, target.subdir, i) for i in target.sources]
         deps +=  srcs
+        if target.build_always:
+            deps.append('PHONY')
         elem = NinjaBuildElement(ofilenames, 'CUSTOM_COMMAND', deps)
         cmd = []
         for i in target.command:
@@ -303,6 +307,7 @@ class NinjaBackend(backends.Backend):
                target.subdir, target.command] + target.args
         elem.add_item('COMMAND', cmd)
         elem.add_item('description', 'Running external command %s.' % target.name)
+        elem.add_item('pool', 'console')
         elem.write(outfile)
         self.processed_targets[target.name] = True
 
@@ -368,6 +373,7 @@ class NinjaBackend(backends.Backend):
         elem.add_dep('all')
         elem.add_item('DESC', 'Installing files.')
         elem.add_item('COMMAND', [sys.executable, install_script, install_data_file])
+        elem.add_item('pool', 'console')
         self.generate_target_install(d)
         self.generate_header_install(d)
         self.generate_man_install(d)
@@ -375,6 +381,7 @@ class NinjaBackend(backends.Backend):
         self.generate_po_install(d, elem)
         self.generate_pkgconfig_install(d)
         self.generate_custom_install_script(d)
+        self.generate_subdir_install(d)
         elem.write(outfile)
 
         ofile = open(install_data_file, 'wb')
@@ -409,15 +416,14 @@ class NinjaBackend(backends.Backend):
                 d.targets.append(i)
 
     def generate_pkgconfig_install(self, d):
-        pkgroot = os.path.join(self.environment.coredata.prefix,
-                               self.environment.coredata.libdir, 'pkgconfig')
+        pkgroot = os.path.join(self.environment.coredata.libdir, 'pkgconfig')
 
         for p in self.build.pkgconfig_gens:
             pcfile = p.filebase + '.pc'
             srcabs = os.path.join(self.environment.get_scratch_dir(),
                                   pcfile)
-            dstabs = os.path.join(pkgroot, pcfile)
-            i = [srcabs, dstabs]
+            dstrel = os.path.join(pkgroot, pcfile)
+            i = [srcabs, dstrel]
             d.man.append(i)
 
     def generate_custom_install_script(self, d):
@@ -431,9 +437,9 @@ class NinjaBackend(backends.Backend):
         for h in headers:
             outdir = h.get_custom_install_dir()
             if outdir is None:
-                outdir = os.path.join(incroot, h.get_subdir())
+                outdir = os.path.join(incroot, h.get_install_subdir())
             for f in h.get_sources():
-                abspath = os.path.join(self.environment.get_source_dir(), f) # FIXME
+                abspath = os.path.join(self.environment.get_source_dir(), h.get_source_subdir(), f)
                 i = [abspath, outdir]
                 d.headers.append(i)
 
@@ -446,7 +452,7 @@ class NinjaBackend(backends.Backend):
                 subdir = m.get_custom_install_dir()
                 if subdir is None:
                     subdir = os.path.join(manroot, 'man' + num)
-                srcabs = os.path.join(self.environment.get_source_dir(), f)
+                srcabs = os.path.join(self.environment.get_source_dir(), m.get_source_subdir(), f)
                 dstabs = os.path.join(subdir, f + '.gz')
                 i = [srcabs, dstabs]
                 d.man.append(i)
@@ -455,15 +461,23 @@ class NinjaBackend(backends.Backend):
         dataroot = self.environment.get_datadir()
         data = self.build.get_data()
         for de in data:
-            subdir = de.get_custom_install_dir()
-            if subdir is None:
-                subdir = os.path.join(dataroot, de.get_subdir())
+            subdir = de.get_install_dir()
             for f in de.get_sources():
-                srcabs = os.path.join(self.environment.get_source_dir(), f)
+                if de.in_sourcetree:
+                    srcprefix = self.environment.get_source_dir()
+                else:
+                    srcprefix = self.environment.get_build_dir()
+                srcabs = os.path.join(srcprefix, de.get_source_subdir(), f)
                 dstabs = os.path.join(subdir, f)
                 i = [srcabs, dstabs]
                 d.data.append(i)
 
+    def generate_subdir_install(self, d):
+        for sd in self.build.get_install_subdirs():
+            src_dir = os.path.join(self.environment.get_source_dir(), sd.source_subdir, sd.installable_subdir)
+            dst_dir = os.path.join(self.environment.get_prefix(), sd.install_dir)
+            d.install_subdirs.append([src_dir, dst_dir])
+
     def generate_tests(self, outfile):
         self.serialise_tests()
         valgrind = environment.find_valgrind()
@@ -471,39 +485,19 @@ class NinjaBackend(backends.Backend):
         test_script = os.path.join(script_root, 'meson_test.py')
         test_data = os.path.join(self.environment.get_scratch_dir(), 'meson_test_setup.dat')
         cmd = [sys.executable, test_script, test_data]
-        elem = NinjaBuildElement('test', 'CUSTOM_COMMAND', 'all')
+        elem = NinjaBuildElement('test', 'CUSTOM_COMMAND', ['all', 'PHONY'])
         elem.add_item('COMMAND', cmd)
         elem.add_item('DESC', 'Running test suite.')
+        elem.add_item('pool', 'console')
         elem.write(outfile)
 
         if valgrind:
-            velem = NinjaBuildElement('test-valgrind', 'CUSTOM_COMMAND', 'all')
+            velem = NinjaBuildElement('test-valgrind', 'CUSTOM_COMMAND', ['all', 'PHONY'])
             velem.add_item('COMMAND', cmd + ['--wrapper=' + valgrind])
             velem.add_item('DESC', 'Running test suite under Valgrind.')
+            velem.add_item('pool', 'console')
             velem.write(outfile)
 
-    def generate_dep_gen_rules(self, outfile):
-        outfile.write('# Rules for external dependency generators.\n\n')
-        processed = {}
-        for dep in self.environment.coredata.deps.values():
-            name = dep.get_name()
-            if name in processed:
-                continue
-            processed[name] = True
-            for rule in dep.get_generate_rules():
-                outfile.write('rule %s\n' % rule.name)
-                command = ' '.join([ninja_quote(x) for x in rule.cmd_list])
-                command = command.replace('@INFILE@', '$in').replace('@OUTFILE@', '$out')
-                command = command.replace('@SOURCE_ROOT@', self.environment.get_source_dir())
-                command = command.replace('@BUILD_ROOT@', self.environment.get_build_dir())
-                outfile.write(' command = %s\n' % command)
-                desc = rule.description.replace('@INFILE@', '$in')
-                outfile.write(' description = %s\n' % desc)
-                if rule.src_keyword in self.dep_rules:
-                    raise InvalidArguments('Multiple rules for keyword %s.' % rule.src_keyword)
-                self.dep_rules[rule.src_keyword] = rule
-            outfile.write('\n')
-
     def generate_rules(self, outfile):
         outfile.write('# Rules for compiling.\n\n')
         self.generate_compile_rules(outfile)
@@ -512,7 +506,6 @@ class NinjaBackend(backends.Backend):
             self.generate_static_link_rules(True, outfile)
         self.generate_static_link_rules(False, outfile)
         self.generate_dynamic_link_rules(outfile)
-        self.generate_dep_gen_rules(outfile)
         outfile.write('# Other rules\n\n')
         outfile.write('rule CUSTOM_COMMAND\n')
         outfile.write(' command = $COMMAND\n')
@@ -547,6 +540,11 @@ class NinjaBackend(backends.Backend):
         outfile.write(desc)
         outfile.write('\n')
 
+    def generate_phony(self, outfile):
+        outfile.write('# Phony build target, always out of date\n')
+        outfile.write('build PHONY: phony\n')
+        outfile.write('\n')
+
     def generate_jar_target(self, target, outfile):
         fname = target.get_filename()
         subdir = target.get_subdir()
@@ -634,8 +632,7 @@ class NinjaBackend(backends.Backend):
     def generate_single_java_compile(self, subdir, src, target, compiler, outfile):
         buildtype = self.environment.coredata.buildtype
         args = []
-        if buildtype == 'debug':
-            args += compiler.get_debug_args()
+        args += compiler.get_buildtype_args(self.environment.coredata.buildtype)
         args += compiler.get_output_args(self.get_target_private_dir(target))
         rel_src = os.path.join(self.build_to_src, subdir, src)
         plain_class_path = src[:-4] + 'class'
@@ -852,7 +849,7 @@ class NinjaBackend(backends.Backend):
         outfile.write('\n')
 
     def generate_fortran_dep_hack(self, outfile):
-        if environment.is_windows():
+        if mesonlib.is_windows():
             cmd = 'cmd /C ""'
         else:
             cmd = 'true'
@@ -892,9 +889,14 @@ rule FORTRAN_DEP_HACK
             crstr = ''
         rule = 'rule %s%s_COMPILER\n' % (langname, crstr)
         depargs = compiler.get_dependency_gen_args('$out', '$DEPFILE')
+        quoted_depargs = []
+        for d in depargs:
+            if d != '$out' and d != '$in':
+                d = qstr % d
+            quoted_depargs.append(d)
         command = " command = %s $ARGS %s %s %s $in\n" % \
             (' '.join(compiler.get_exelist()),\
-             ' '.join([qstr % d for d in depargs]),\
+             ' '.join(quoted_depargs),\
              ' '.join(compiler.get_output_args('$out')),\
              ' '.join(compiler.get_compile_only_args()))
         description = ' description = Compiling %s object $out\n' % langname
@@ -918,13 +920,18 @@ rule FORTRAN_DEP_HACK
             crstr = ''
         rule = 'rule %s%s_PCH\n' % (langname, crstr)
         depargs = compiler.get_dependency_gen_args('$out', '$DEPFILE')
+        quoted_depargs = []
+        for d in depargs:
+            if d != '$out' and d != '$in':
+                d = qstr % d
+            quoted_depargs.append(d)
         if compiler.get_id() == 'msvc':
             output = ''
         else:
             output = ' '.join(compiler.get_output_args('$out'))
         command = " command = %s $ARGS %s %s %s $in\n" % \
             (' '.join(compiler.get_exelist()),\
-             ' '.join([qstr % d for d in depargs]),\
+             ' '.join(quoted_depargs),\
              output,\
              ' '.join(compiler.get_compile_only_args()))
         description = ' description = Precompiling header %s\n' % '$in'
@@ -987,7 +994,7 @@ rule FORTRAN_DEP_HACK
             base_args = generator.get_arglist()
             for i in range(len(infilelist)):
                 if len(generator.outputs) == 1:
-                    sole_output = os.path.join(self.get_target_private_dir(target), outfilelist[0])
+                    sole_output = os.path.join(self.get_target_private_dir(target), outfilelist[i])
                 else:
                     sole_output = ''
                 curfile = infilelist[i]
@@ -1072,6 +1079,16 @@ rule FORTRAN_DEP_HACK
         compiler = self.get_compiler_for_source(src)
         commands = self.generate_basic_compiler_args(target, compiler)
         commands += compiler.get_include_args(self.get_target_private_dir(target))
+        curdir = target.get_subdir()
+        tmppath = os.path.normpath(os.path.join(self.build_to_src, curdir))
+        commands += compiler.get_include_args(tmppath)
+        if curdir ==  '':
+            curdir = '.'
+        commands += compiler.get_include_args(curdir)
+        for d in target.external_deps:
+            if d.need_threads():
+                commands += compiler.thread_flags()
+                break
         if isinstance(src, RawFilename):
             rel_src = src.fname
         elif is_generated:
@@ -1256,15 +1273,18 @@ rule FORTRAN_DEP_HACK
         else:
             dependencies = target.get_dependencies()
         commands += self.build_target_link_arguments(linker, dependencies)
+        for d in target.external_deps:
+            if d.need_threads():
+                commands += linker.thread_link_flags()
         commands += target.link_args
         # External deps must be last because target link libraries may depend on them.
         if not(isinstance(target, build.StaticLibrary)):
             for dep in target.get_external_deps():
                 commands += dep.get_link_args()
-        for d in target.get_dependencies():
-            if isinstance(d, build.StaticLibrary):
-                for dep in d.get_external_deps():
-                    commands += dep.get_link_args()
+            for d in target.get_dependencies():
+                if isinstance(d, build.StaticLibrary):
+                    for dep in d.get_external_deps():
+                        commands += dep.get_link_args()
         commands += linker.build_rpath_args(self.environment.get_build_dir(),\
                                             target.get_rpaths(), target.install_rpath)
         if self.environment.coredata.coverage:
@@ -1298,14 +1318,14 @@ rule FORTRAN_DEP_HACK
         elem.write(outfile)
 
     def generate_gcov_clean(self, outfile):
-            gcno_elem = NinjaBuildElement('clean-gcno', 'CUSTOM_COMMAND', '')
+            gcno_elem = NinjaBuildElement('clean-gcno', 'CUSTOM_COMMAND', 'PHONY')
             script_root = self.environment.get_script_dir()
             clean_script = os.path.join(script_root, 'delwithsuffix.py')
             gcno_elem.add_item('COMMAND', [sys.executable, clean_script, '.', 'gcno'])
             gcno_elem.add_item('description', 'Deleting gcno files')
             gcno_elem.write(outfile)
 
-            gcda_elem = NinjaBuildElement('clean-gcda', 'CUSTOM_COMMAND', '')
+            gcda_elem = NinjaBuildElement('clean-gcda', 'CUSTOM_COMMAND', 'PHONY')
             script_root = self.environment.get_script_dir()
             clean_script = os.path.join(script_root, 'delwithsuffix.py')
             gcda_elem.add_item('COMMAND', [sys.executable, clean_script, '.', 'gcda'])
@@ -1335,24 +1355,11 @@ rule FORTRAN_DEP_HACK
                 outfilename = os.path.join(self.get_target_private_dir(target), outname)
                 infilename = os.path.join(self.build_to_src, target.get_source_subdir(), src)
                 elem = NinjaBuildElement(outfilename, rule.name, infilename)
-                if rule.name == 'rc_compile':
-                    elem.add_item('rcc_args', ['--name', basename])
-                if rule.name == 'moc_hdr_compile' or rule.name == 'moc_src_compile':
-                    elem.add_item('mocargs', ['-I', target.subdir])
                 elem.write(outfile)
                 if self.is_compilable_file(outfilename):
-                    if rule.name == 'moc_hdr_compile' or rule.name == 'moc_src_compile':
-                        manual_mocs = target.get_original_kwargs().get('manual_moc_include', [])
-                        if src in manual_mocs:
-                            other_deps.append(outfilename)
-                        else:
-                            src_deps.append(outfilename)
-                    else:
-                        src_deps.append(outfilename)
+                    src_deps.append(outfilename)
                 else:
                     other_deps.append(outfilename)
-                if rule.name == 'moc_src_compile': #HACK
-                    src_deps.append(infilename)
         return (src_deps, other_deps)
 
     def generate_ending(self, outfile):
@@ -1367,8 +1374,8 @@ rule FORTRAN_DEP_HACK
 
         ninja_command = environment.detect_ninja()
         if ninja_command is None:
-            raise RuntimeError('Could not detect ninja command')
-        elem = NinjaBuildElement('clean', 'CUSTOM_COMMAND', '')
+            raise MesonException('Could not detect ninja command')
+        elem = NinjaBuildElement('clean', 'CUSTOM_COMMAND', 'PHONY')
         elem.add_item('COMMAND', [ninja_command, '-t', 'clean'])
         elem.add_item('description', 'Cleaning')
         if self.environment.coredata.coverage:
@@ -1390,6 +1397,7 @@ rule FORTRAN_DEP_HACK
             if os.path.isfile(fname):
                 deps.append(os.path.join(self.build_to_src, sp, 'meson_options.txt'))
         elem = NinjaBuildElement('build.ninja', 'REGENERATE_BUILD', deps)
+        elem.add_item('pool', 'console')
         elem.write(outfile)
 
         elem = NinjaBuildElement(deps, 'phony', '')
