@@ -1,4 +1,4 @@
-# Copyright 2013-2014 Jussi Pakkanen
+# Copyright 2013-2014 The Meson development team
 
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,9 +14,9 @@
 
 import mparser
 import coredata
-import os
+import os, re
 
-forbidden_option_names = {'type': True,
+forbidden_option_names = {'buildtype': True,
                           'strip': True,
                           'coverage': True,
                           'pch': True,
@@ -34,6 +34,8 @@ forbidden_option_names = {'type': True,
 class OptionException(coredata.MesonException):
     pass
 
+optname_regex = re.compile('[^a-zA-Z0-9_-]')
+
 class UserOption:
     def __init__(self, kwargs):
         super().__init__()
@@ -166,10 +168,12 @@ class OptionInterpreter:
         opt_name = posargs[0]
         if not isinstance(opt_name, str):
             raise OptionException('Positional argument must be a string.')
+        if optname_regex.search(opt_name) is not None:
+            raise OptionException('Option names can only contain letters, numbers or dashes.')
         if opt_name in forbidden_option_names:
             raise OptionException('Option name %s is reserved.' % opt_name)
         if self.subproject != '':
-            opt_name = self.subproject + '-' + opt_name
+            opt_name = self.subproject + ':' + opt_name
         opt = option_types[opt_type](kwargs)
         if opt.description == '':
             opt.description = opt_name
