@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 
-# Copyright 2012-2014 Jussi Pakkanen
+# Copyright 2012-2015 The Meson development team
 
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,43 +15,67 @@
 # limitations under the License.
 
 from glob import glob
-import os, subprocess, shutil, sys, platform
+import os, subprocess, shutil, sys, platform, signal
 import environment
-from environment import is_windows
+import mesonlib
+import argparse
+import xml.etree.ElementTree as ET
+import time
+
+from meson import backendlist
 
 passing_tests = 0
 failing_tests = 0
+skipped_tests = 0
 print_debug = 'MESON_PRINT_TEST_OUTPUT' in os.environ
 
 test_build_dir = 'work area'
 install_dir = os.path.join(os.path.split(os.path.abspath(__file__))[0], 'install dir')
 meson_command = './meson.py'
 
+class StopException(Exception):
+    def __init__(self):
+        super(Exception, self).__init__('Stopped by user')
+
+stop = False
+def stop_handler(signal, frame):
+    global stop
+    stop = True
+signal.signal(signal.SIGINT, stop_handler)
+signal.signal(signal.SIGTERM, stop_handler)
+
 #unity_flags = ['--unity']
 unity_flags = []
-msbuild_exe = shutil.which('msbuild')
 
-if msbuild_exe is not None:
-    backend_flags = ['--backend=vs2010']
-    compile_commands = ['msbuild']
-    test_commands = ['msbuild', 'RUN_TESTS.vcxproj']
-    install_commands = []
-elif environment.is_osx():
-    backend_flags = ['--backend=xcode']
-    compile_commands = ['xcodebuild']
-    test_commands = ['xcodebuild', '-target', 'RUN_TESTS']
-    install_commands = []
-else:
-    backend_flags = []
-    ninja_command = environment.detect_ninja()
-    if ninja_command is None:
-        raise RuntimeError('Could not find Ninja executable.')
-    if print_debug:
-        compile_commands = [ninja_command, '-v']
+backend_flags = None
+compile_commands = None
+test_commands = None
+install_commands = None
+
+def setup_commands(backend):
+    global backend_flags, compile_commands, test_commands, install_commands
+    msbuild_exe = shutil.which('msbuild')
+    if backend == 'vs2010' or (backend is None and msbuild_exe is not None):
+        backend_flags = ['--backend=vs2010']
+        compile_commands = ['msbuild']
+        test_commands = ['msbuild', 'RUN_TESTS.vcxproj']
+        install_commands = []
+    elif backend == 'xcode' or (backend is None and mesonlib.is_osx()):
+        backend_flags = ['--backend=xcode']
+        compile_commands = ['xcodebuild']
+        test_commands = ['xcodebuild', '-target', 'RUN_TESTS']
+        install_commands = []
     else:
-        compile_commands = [ninja_command]
-    test_commands = [ninja_command, 'test']
-    install_commands = [ninja_command, 'install']
+        backend_flags = []
+        ninja_command = environment.detect_ninja()
+        if ninja_command is None:
+            raise RuntimeError('Could not find Ninja executable.')
+        if print_debug:
+            compile_commands = [ninja_command, '-v']
+        else:
+            compile_commands = [ninja_command]
+        test_commands = [ninja_command, 'test']
+        install_commands = [ninja_command, 'install']
 
 def platform_fix_filename(fname):
     if platform.system() == 'Darwin':
@@ -92,9 +116,8 @@ def validate_install(srcdir, installdir):
         return 'Found extra file %s.' % fname
     return ''
 
-def run_and_log(logfile, testdir, should_succeed=True):
-    global passing_tests, failing_tests
-    (msg, stdo, stde) = run_test(testdir, should_succeed)
+def log_text_file(logfile, testdir, msg, stdo, stde):
+    global passing_tests, failing_tests, stop
     if msg != '':
         print('Fail:', msg)
         failing_tests += 1
@@ -109,6 +132,8 @@ def run_and_log(logfile, testdir, should_succeed=True):
     if print_debug:
         print(stdo)
         print(stde, file=sys.stderr)
+    if stop:
+        raise StopException()
 
 def run_test(testdir, should_succeed):
     global compile_commands
@@ -117,7 +142,7 @@ def run_test(testdir, should_succeed):
     os.mkdir(test_build_dir)
     os.mkdir(install_dir)
     print('Running test: ' + testdir)
-    gen_command = [sys.executable, meson_command, '--prefix', install_dir, testdir, test_build_dir]\
+    gen_command = [sys.executable, meson_command, '--prefix', '/usr', '--libdir', 'lib', testdir, test_build_dir]\
         + unity_flags + backend_flags
     p = subprocess.Popen(gen_command, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
     (stdo, stde) = p.communicate()
@@ -152,7 +177,9 @@ def run_test(testdir, should_succeed):
         print("Skipping install test")
         return ('', '', '')
     else:
-        pi = subprocess.Popen(install_commands, cwd=test_build_dir,
+        env = os.environ.copy()
+        env['DESTDIR'] = install_dir
+        pi = subprocess.Popen(install_commands, cwd=test_build_dir, env=env,
                           stdout=subprocess.PIPE, stderr=subprocess.PIPE)
         (o, e) = pi.communicate()
         stdo += o.decode('utf-8')
@@ -168,50 +195,28 @@ def gather_tests(testdir):
     tests = [os.path.join(testdir, t[1]) for t in testlist]
     return tests
 
+def detect_tests_to_run():
+    all_tests = []
+    all_tests.append(('common', gather_tests('test cases/common'), False))
+    all_tests.append(('failing', gather_tests('test cases/failing'), False))
+    all_tests.append(('prebuilt object', gather_tests('test cases/prebuilt object'), False))
+
+    all_tests.append(('platform-osx', gather_tests('test cases/osx'), False if mesonlib.is_osx() else True))
+    all_tests.append(('platform-windows', gather_tests('test cases/windows'), False if mesonlib.is_windows() else True))
+    all_tests.append(('platform-linux', gather_tests('test cases/linuxlike'), False if not (mesonlib.is_osx() or mesonlib.is_windows()) else True))
+    all_tests.append(('framework', gather_tests('test cases/frameworks'), False if not mesonlib.is_osx() and not mesonlib.is_windows() else True))
+    all_tests.append(('java', gather_tests('test cases/java'), False if not mesonlib.is_osx() and shutil.which('javac') else True))
+    all_tests.append(('C#', gather_tests('test cases/csharp'), False if shutil.which('mcs') else True))
+    all_tests.append(('vala', gather_tests('test cases/vala'), False if shutil.which('valac') else True))
+    all_tests.append(('rust', gather_tests('test cases/rust'), False if shutil.which('rustc') else True))
+    all_tests.append(('objective c', gather_tests('test cases/objc'), False if not mesonlib.is_windows() else True))
+    all_tests.append(('fortran', gather_tests('test cases/fortran'), False if shutil.which('gfortran') else True))
+    return all_tests
+
 def run_tests():
+    all_tests = detect_tests_to_run()
     logfile = open('meson-test-run.txt', 'w')
-    commontests = gather_tests('test cases/common')
-    failtests = gather_tests('test cases/failing')
-    objtests = gather_tests('test cases/prebuilt object')
-    if environment.is_linux():
-        cpuid = platform.machine()
-        if cpuid != 'x86_64' and cpuid != 'i386' and cpuid != 'i686':
-            # Don't have a prebuilt object file for those so skip.
-            objtests = []
-    if environment.is_osx():
-        platformtests = gather_tests('test cases/osx')
-    elif environment.is_windows():
-        platformtests = gather_tests('test cases/windows')
-    else:
-        platformtests = gather_tests('test cases/linuxlike')
-    if not environment.is_osx() and not environment.is_windows():
-        frameworktests = gather_tests('test cases/frameworks')
-    else:
-        frameworktests = []
-    if not environment.is_osx() and shutil.which('javac'):
-        javatests = gather_tests('test cases/java')
-    else:
-        javatests = []
-    if shutil.which('mcs'):
-        cstests = gather_tests('test cases/csharp')
-    else:
-        cstests = []
-    if shutil.which('valac'):
-        valatests = gather_tests('test cases/vala')
-    else:
-        valatests = []
-    if shutil.which('rustc'):
-        rusttests = gather_tests('test cases/rust')
-    else:
-        rusttests = []
-    if not environment.is_windows():
-        objctests = gather_tests('test cases/objc')
-    else:
-        objctests = []
-    if shutil.which('gfortran'):
-        fortrantests = gather_tests('test cases/fortran')
-    else:
-        fortrantests = []
+    junit_root = ET.Element('testsuites')
     try:
         os.mkdir(test_build_dir)
     except OSError:
@@ -220,55 +225,39 @@ def run_tests():
         os.mkdir(install_dir)
     except OSError:
         pass
-    print('\nRunning common tests.\n')
-    [run_and_log(logfile, t) for t in commontests]
-    print('\nRunning failing tests.\n')
-    [run_and_log(logfile, t, False) for t in failtests]
-    if len(objtests) > 0:
-        print('\nRunning object inclusion tests.\n')
-        [run_and_log(logfile, t) for t in objtests]
-    else:
-        print('\nNo object inclusion tests.\n')
-    if len(platformtests) > 0:
-        print('\nRunning platform dependent tests.\n')
-        [run_and_log(logfile, t) for t in platformtests]
-    else:
-        print('\nNo platform specific tests.\n')
-    if len(frameworktests) > 0:
-        print('\nRunning framework tests.\n')
-        [run_and_log(logfile, t) for t in frameworktests]
-    else:
-        print('\nNo framework tests on this platform.\n')
-    if len(javatests) > 0:
-        print('\nRunning java tests.\n')
-        [run_and_log(logfile, t) for t in javatests]
-    else:
-        print('\nNot running Java tests.\n')
-    if len(cstests) > 0:
-        print('\nRunning C# tests.\n')
-        [run_and_log(logfile, t) for t in cstests]
-    else:
-        print('\nNot running C# tests.\n')
-    if len(valatests) > 0:
-        print('\nRunning Vala tests.\n')
-        [run_and_log(logfile, t) for t in valatests]
-    else:
-        print('\nNot running Vala tests.\n')
-    if len(rusttests) > 0:
-        print('\nRunning Rust tests.\n')
-        [run_and_log(logfile, t) for t in rusttests]
-    else:
-        print('\nNot running Rust tests.\n')
-    if len(objctests) > 0:
-        print('\nRunning Objective C tests.\n')
-        [run_and_log(logfile, t) for t in objctests]
-    else:
-        print('\nNo Objective C tests on this platform.\n')
-    if len(fortrantests) > 0:
-        print('\nRunning Fortran tests.\n')
-        [run_and_log(logfile, t) for t in fortrantests]
-    else:
-        print('\nNo Fortran tests on this platform.\n')
+
+    for name, test_cases, skipped in all_tests:
+        current_suite = ET.SubElement(junit_root, 'testsuite', {'name' : name, 'tests' : str(len(test_cases))})
+        if skipped:
+            print('\nNot running %s tests.\n' % name)
+        else:
+            print('\nRunning %s tests.\n' % name)
+        for t in test_cases:
+            # Jenkins screws us over by automatically sorting test cases by name
+            # and getting it wrong by not doing logical number sorting.
+            (testnum, testbase) = os.path.split(t)[-1].split(' ', 1)
+            testname = '%.3d %s' % (int(testnum), testbase)
+            if skipped:
+                current_test = ET.SubElement(current_suite, 'testcase', {'name' : testname,
+                                                                         'classname' : name})
+                ET.SubElement(current_test, 'skipped', {})
+                global skipped_tests
+                skipped_tests += 1
+            else:
+                ts = time.time()
+                (msg, stdo, stde) = run_test(t, name != 'failing')
+                te = time.time()
+                log_text_file(logfile, t, msg, stdo, stde)
+                current_test = ET.SubElement(current_suite, 'testcase', {'name' : testname,
+                                                                         'classname' : name,
+                                                                         'time' : '%.3f' % (te - ts)})
+                if msg != '':
+                    ET.SubElement(current_test, 'failure', {'message' : msg})
+                stdoel = ET.SubElement(current_test, 'system-out')
+                stdoel.text = stdo
+                stdeel = ET.SubElement(current_test, 'system-err')
+                stdeel.text = stde
+    ET.ElementTree(element=junit_root).write('meson-test-run.xml', xml_declaration=True, encoding='UTF-8')
 
 def check_file(fname):
     linenum = 1
@@ -295,7 +284,7 @@ def generate_prebuilt_object():
         objectfile = objectbase + 'obj'
         cmd = ['cl', '/nologo', '/Fo'+objectfile, '/c', source]
     else:
-        if is_windows():
+        if mesonlib.is_windows():
             objectfile = objectbase + 'obj'
         else:
             objectfile = objectbase + 'o'
@@ -304,14 +293,24 @@ def generate_prebuilt_object():
     return objectfile
 
 if __name__ == '__main__':
+    parser = argparse.ArgumentParser(description="Run the test suite of Meson.")
+    parser.add_argument('--backend', default=None, dest='backend',
+                      choices = backendlist)
+    options = parser.parse_args()
+    setup_commands(options.backend)
+
     script_dir = os.path.split(__file__)[0]
     if script_dir != '':
         os.chdir(script_dir)
     check_format()
     pbfile = generate_prebuilt_object()
-    run_tests()
+    try:
+        run_tests()
+    except StopException:
+        pass
     os.unlink(pbfile)
     print('\nTotal passed tests:', passing_tests)
     print('Total failed tests:', failing_tests)
+    print('Total skipped tests:', skipped_tests)
     sys.exit(failing_tests)
 
